package algorithm;

import java.text.NumberFormat;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang3.mutable.MutableDouble;
import org.apache.commons.lang3.mutable.MutableInt;

import progOpt.TimeLogger;
import algorithm.dataStructures.CompactDegreeSequence;
import algorithm.dataStructures.DegreeSequence;
import algorithm.dataStructures.GraphInterface;
import algorithm.dataStructures.Timer;
import algorithm.graphRealizer.CannotRealize;
import algorithm.graphRealizer.GraphRealizer;
import benchmarks.FKDA;
import dataStructures.IntTupel;
import dataStructures.Store;

public class UpperBound
{
	public static final MutableDouble FastUpperBound_weightFrac1 = new MutableDouble(0.4d);// [0,1]

	public static final MutableInt HIGH_QUALITY_IN_EACH_STEP = new MutableInt(5);

	public static NumberFormat largeIntFormater = NumberFormat.getInstance();

	public static final MutableDouble MAGIC_CONSTANT_FOR_COST_LMINUS1 = new MutableDouble(1.2);// [1,3]

	public static final MutableDouble MAGIC_CONSTANT_FOR_COST_LMINUS1_EMPTY = new MutableDouble(0.9);// [0,1]
	/**
	 * Determines how much the heuristic prefers to increase degrees up to the last full block (the smaller the value the more it is preferred)
	 * instead of filling it up by lower degrees.
	 */
	public static final MutableDouble MAGIC_UPPERBOUND_HEURISTIC_CONSTANT = new MutableDouble(1.1d); // [0,3]
	public static int printRealizerParams = 0;

	public static boolean printUpperBoundInHeuristicsSize = false;

	public static final MutableInt REALIZE_SOLUTIONS_WEIGHT_OF_FIRST = new MutableInt(4);

	public static final MutableInt StepIncrease_Magic_Const = new MutableInt(11); // [0,20]

	public static final MutableDouble StepIncrease_Magic_Const2 = new MutableDouble(0.5); // [0,5]

	public static final MutableInt StepIncrease_Magic_Const3 = new MutableInt(15);

	public static int getClusteringHeuristicUpperBound(final int k, final GraphInterface graph)
	{
		final FKDA fkda = new FKDA();
		final int size = Math.min(Math.min(fkda.fkda(graph, k, FKDA.Heuristic.RANDOM).size(), fkda.fkda(graph, k, FKDA.Heuristic.BACKWARD).size()),
				fkda.fkda(graph, k, FKDA.Heuristic.FORWARD).size());
		return size * 2;
	}

	/**
	 * Returns an upperbound on the k-anon. cost of a degree sequence (does not use our dynamic programming, but should be really fast).
	 * 
	 * @param seq
	 * @param k
	 * @return
	 */
	public static Store<Integer, CompactDegreeSequence> getFastUpperBoundForKAnon(final CompactDegreeSequence seq, final int k)
	{
		final float magic_Constant = UpperBound.MAGIC_UPPERBOUND_HEURISTIC_CONSTANT.floatValue();

		if (seq.getTotalNumberOfVertices() < k)
			throw new IllegalArgumentException("k is larger than n");

		final int arr[] = new int[seq.getMaxDegree()+1];

		int lastFullBag = -1;
		int currentBag = seq.getMaxDegree();
		while (currentBag >= 0)
		{
			if (seq.get(currentBag) > 0)
			{
				int l = currentBag;
				{// l will be the greatest index <=currentBag such that in
					// [l...currentbag] there are at least k vertices
					while ((l >= 0) && (seq.getNumberOfVerticesInBags(l, currentBag) < k))
					{
						l--;
					}
				}
				if (l < 0)
				{// [l...currentbag] contains less than k vertices > increase
					// all of them to lastFullBag
					if (lastFullBag < 0)
						throw new IllegalArgumentException();
					arr[lastFullBag] += seq.getNumberOfVerticesInBags(0, currentBag);
					currentBag = -1;
				} else if (l < currentBag)
				{// [l...currentbag] contains at least k vertices
					final int costCurrentUpToLast = seq.get(currentBag) * (lastFullBag - currentBag);
					final int neededVerticesFromBlockL = k - seq.getNumberOfVerticesInBags(l + 1, currentBag);

					int costLTillCurrentUpToCurrent = (seq.getNumberOfVerticesInBags(l + 1, currentBag) * currentBag)
							- seq.getDegreeSumsInBags(l + 1, currentBag);
					costLTillCurrentUpToCurrent += neededVerticesFromBlockL * (currentBag - l);

					final double costForCurrentMinus1 = (((currentBag - 1) >= 0) && (seq.get(currentBag - 1) > 0) && (seq.get(currentBag - 1) < k)) ? UpperBound.MAGIC_CONSTANT_FOR_COST_LMINUS1
							.doubleValue() : UpperBound.MAGIC_CONSTANT_FOR_COST_LMINUS1_EMPTY.doubleValue();
					final double costFrac1 = (magic_Constant * costCurrentUpToLast * costForCurrentMinus1) / costLTillCurrentUpToCurrent;// differences
					// in cost
					final double costFrac2 = (lastFullBag - currentBag) / (currentBag - l);// differences in max raise

					if ((lastFullBag >= 0)
							&& (((UpperBound.FastUpperBound_weightFrac1.doubleValue() * costFrac1) + ((1d - UpperBound.FastUpperBound_weightFrac1
									.doubleValue()) * costFrac2)) <= 1d))
					{// increase currentBag to lastFullBag
						arr[lastFullBag] += seq.get(currentBag);
						currentBag--;
					} else if ((seq.get(l) - neededVerticesFromBlockL) < k)
					{// increase all vertices in [l...currentBag] up to lastFullBag
						arr[currentBag] = seq.getNumberOfVerticesInBags(l, currentBag);
						lastFullBag = currentBag;
						currentBag = l - 1;
					} else
					{// increase all vertices in [l...currentBag] up to lastFullBag; only the needed number of vertices get removed from bag l
						arr[currentBag] = seq.getNumberOfVerticesInBags(l + 1, currentBag) + neededVerticesFromBlockL;
						arr[l] = seq.get(l) - neededVerticesFromBlockL;
						lastFullBag = l;
						currentBag = l - 1;
					}

				} else
				// l=currentbag > currentBag contains at least k vertices
				{
					arr[currentBag] = seq.get(currentBag);
					lastFullBag = currentBag;
					currentBag--;
				}
			} else
			{// currentBag doesn't contain any vertex
				currentBag--;
			}
		}
		final CompactDegreeSequence compactDegreeSequence = new CompactDegreeSequence(arr, false);
		return new Store<Integer, CompactDegreeSequence>(compactDegreeSequence.difference(seq).getTotalDegreeSum(), compactDegreeSequence);
	}

	public static int getFastUpperBoundForKAnonAndRealizable(final GraphInterface graph, final int k, final int maxTimeInSeconds)
	{
		final Timer timer = new Timer();
		final CompactDegreeSequence origSeq = new CompactDegreeSequence(DegreeSequence.calcDegreeSequence(graph), true);
		final long t = UpperBound.printUpperBoundInHeuristicsSize ? System.currentTimeMillis() : 0;

		final Set<CompactDegreeSequence> allSolutions = new HashSet<CompactDegreeSequence>();
		final Store<Integer, CompactDegreeSequence> upperBoundForKAnon = UpperBound.getFastUpperBoundForKAnon(origSeq, k);
		allSolutions.add(upperBoundForKAnon.getS2());

		if (UpperBound.printUpperBoundInHeuristicsSize)
		{
			System.out.println("fast upper bound:" + UpperBound.largeIntFormater.format(upperBoundForKAnon.getS1()) + " (time:"
					+ TimeLogger.timeFormatter.format((System.currentTimeMillis() - t) / 1000d) + "s)");
		}
		return UpperBound.realizeUpperBoundSolution(graph, k, origSeq, allSolutions, (int) (maxTimeInSeconds - timer.elapsedTimeInSeconds())).getS1();
	}

	/**
	 * Returns an upperbound for k-anonymizing the given graph (checks realizability) using our dynamic programming algorithm.
	 * 
	 * @param graph
	 * @param k
	 * @return
	 */
	public static Store<Integer, Iterable<IntTupel>> getMoreSophisticatedUpperBoundForKAnonAndRealizable(final GraphInterface graph, final int k,
			final int maxTimeInSeconds)
	{
		final Timer timer = new Timer();
		final CompactDegreeSequence origSeq = new CompactDegreeSequence(DegreeSequence.calcDegreeSequence(graph), true);

		final long t = UpperBound.printUpperBoundInHeuristicsSize ? System.currentTimeMillis() : 0;

		final Set<CompactDegreeSequence> allSolutions = new DegreeAnonymizer(graph, origSeq, k, false, true).getAllkAnonRealSequences();

		if ((allSolutions == null) || (allSolutions.size() == 0))
			throw new IllegalAccessError("more sophisticated upper bound computes null solution");

		if (UpperBound.printUpperBoundInHeuristicsSize)
		{
			System.out.println("more sophisticated upper bound:"
					+ UpperBound.largeIntFormater.format(allSolutions.iterator().next().difference(origSeq).getTotalDegreeSum()) + " (time:"
					+ TimeLogger.timeFormatter.format((System.currentTimeMillis() - t) / 1000d) + "s)");
		}
		return UpperBound.realizeUpperBoundSolution(graph, k, origSeq, allSolutions, (int) (maxTimeInSeconds - timer.elapsedTimeInSeconds()));
	}

	/**
	 * Returns an upperbound for k-anonymizing the given graph (checks realizability) using the dynamic programming algorithm of Liu and Terzi.
	 * 
	 * @param graph
	 * @param k
	 * @return
	 */
	public static Store<Integer, Iterable<IntTupel>> getSophisticatedUpperBoundForKAnonAndRealizable(final GraphInterface graph, final int k,
			final int maxTimeInSeconds)
	{
		final Timer timer = new Timer(maxTimeInSeconds);
		final CompactDegreeSequence origSeq = new CompactDegreeSequence(DegreeSequence.calcDegreeSequence(graph), true);

		final long t = UpperBound.printUpperBoundInHeuristicsSize ? System.currentTimeMillis() : 0;

		final LiuTerziDegreeAnonymizer min = new LiuTerziDegreeAnonymizer(origSeq, k);
		final Set<CompactDegreeSequence> allSolutions = min.getMinimumSolutions();

		if ((allSolutions == null) || (allSolutions.size() == 0))
			throw new IllegalAccessError("sophisticated upper bound computes null solution");

		if (UpperBound.printUpperBoundInHeuristicsSize)
		{
			System.out.println("sophisticated upper bound:"
					+ UpperBound.largeIntFormater.format(allSolutions.iterator().next().difference(origSeq).getTotalDegreeSum()) + " (time:"
					+ TimeLogger.timeFormatter.format((System.currentTimeMillis() - t) / 1000d) + "s)");
		}
		final int remainingTime = timer.getRemainingTime();
		if (remainingTime < GraphRealizer.MINIMUM_TIME_FOR_IS_REALIZABLE)
			return new Store<Integer, Iterable<IntTupel>>(Integer.MAX_VALUE, null);
		return UpperBound.realizeUpperBoundSolution(graph, k, origSeq, allSolutions, remainingTime);
	}

	/**
	 * Given a set containing solutions to the problem of k-anonymizing the degree sequence of the given graph it wastes as least (just a heuristic!)
	 * degrees as possible the get a realizable solution. Returns the difference of this realizable solution to the given graph.
	 * 
	 * @param graph
	 * @param k
	 * @param origSeq degree sequence of graph
	 * @param allSolutions
	 * @return solutions size and an realizing edge set
	 */
	public static Store<Integer, Iterable<IntTupel>> realizeUpperBoundSolution(final GraphInterface graph, final int k,
			final CompactDegreeSequence origSeq, final Set<CompactDegreeSequence> allSolutions, final int maxTimeBound)
	{
		if (maxTimeBound < GraphRealizer.MINIMUM_TIME_FOR_IS_REALIZABLE)
			throw new IllegalAccessError("maxTimeBound must be at least " + (GraphRealizer.MINIMUM_TIME_FOR_IS_REALIZABLE));

		final Timer timer = new Timer(maxTimeBound);

		final int solutionDiffToOrig = allSolutions.iterator().next().difference(origSeq).getTotalDegreeSum();

		if ((UpperBound.printRealizerParams > 0) && !UpperBound.printUpperBoundInHeuristicsSize)
		{
			System.out.println("k-anon cost deg seq:" + UpperBound.largeIntFormater.format(solutionDiffToOrig) + " (#solutions:"
					+ allSolutions.size() + ")");
		}

		Store<Integer, Iterable<IntTupel>> bestSolutionsFoundSoFar = new Store<Integer, Iterable<IntTupel>>(Integer.MAX_VALUE, null);

		int solCount = 0;
		for (final CompactDegreeSequence sol : allSolutions)
		{

			if (timer.getRemainingTime() < 0)
				return bestSolutionsFoundSoFar;

			final int remainingTime = timer.getRemainingTime();
			final double remainingTimePerInstance = remainingTime
					/ ((allSolutions.size() - solCount) + (solCount == 0 ? (UpperBound.REALIZE_SOLUTIONS_WEIGHT_OF_FIRST.intValue() - 1) : 0));

			int maxTimeBoundForNextInstance = (int) Math.floor((solCount++ == 0) ? remainingTimePerInstance
					* UpperBound.REALIZE_SOLUTIONS_WEIGHT_OF_FIRST.intValue() : remainingTime);
			if (maxTimeBoundForNextInstance < GraphRealizer.MINIMUM_TIME_FOR_IS_REALIZABLE)
			{
				if (timer.getRemainingTime() < GraphRealizer.MINIMUM_TIME_FOR_IS_REALIZABLE)
					return bestSolutionsFoundSoFar;
				else
				{
					maxTimeBoundForNextInstance = GraphRealizer.MINIMUM_TIME_FOR_IS_REALIZABLE;
				}
			}
			try
			{
				bestSolutionsFoundSoFar = GraphRealizer.isRealizable(graph, origSeq, sol, bestSolutionsFoundSoFar.getS1() - solutionDiffToOrig - 1,
						maxTimeBoundForNextInstance, k);
			} catch (final CannotRealize e)
			{

			}
		}
		return bestSolutionsFoundSoFar;
	}
}
