package algorithm.alternativeRealizer;

import java.util.HashSet;
import java.util.TreeMap;

import dataStructures.Store;

public class DegreesChangeList
{

	private final TreeMap<Integer, Store<Integer, TreeMap<Integer, Integer>>> allIncreases;

	private final HashSet<Integer> degreesRaisedTo;

	public DegreesChangeList()
	{
		// first int: amount of increase; second int: original degree; third
		// int: amount of vertices
		// of that type
		this.allIncreases = new TreeMap<Integer, Store<Integer, TreeMap<Integer, Integer>>>();
		this.degreesRaisedTo = new HashSet<Integer>();
	}

	public void add(final int increase, final int originalDegree, final int amountOfVertices)
	{
		this.degreesRaisedTo.add(originalDegree + increase);
		if (this.allIncreases.containsKey(increase))
		{
			final Store<Integer, TreeMap<Integer, Integer>> tmp = this.allIncreases.get(increase);
			tmp.setS1(tmp.getS1() + amountOfVertices);
			if (tmp.getS2().containsKey(originalDegree))
			{
				tmp.getS2().put(originalDegree, tmp.getS2().get(originalDegree) + amountOfVertices);
			} else
			{
				tmp.getS2().put(originalDegree, amountOfVertices);
			}
		} else
		{
			final Store<Integer, TreeMap<Integer, Integer>> tmp = new Store<Integer, TreeMap<Integer, Integer>>();
			tmp.setS1(amountOfVertices);
			tmp.setS2(new TreeMap<Integer, Integer>());
			tmp.getS2().put(originalDegree, amountOfVertices);
			this.allIncreases.put(increase, tmp);
		}
	}

	public Store<Integer, TreeMap<Integer, Integer>> getAll(final int increase)
	{
		return this.allIncreases.get(increase);
	}

	public int getAmount(final int increase, final int originalDegree)
	{
		if (this.allIncreases.containsKey(increase) && this.allIncreases.get(increase).getS2().containsKey(originalDegree))
			return this.allIncreases.get(increase).getS2().get(originalDegree);
		else
			return 0;
	}

	public HashSet<Integer> getDegreeRaisedTo()
	{
		return this.degreesRaisedTo;
	}

	public int getNoOfIncreasedVertices(final int increase)
	{
		if (this.allIncreases.containsKey(increase))
			return this.allIncreases.get(increase).getS1();
		else
			return 0;
	}

	public TreeMap<Integer, Store<Integer, TreeMap<Integer, Integer>>> getTreeMap()
	{
		return this.allIncreases;
	}

	@Override
	public String toString()
	{
		return this.allIncreases.toString();
	}

}
