package algorithm.dataStructures;

import java.util.Arrays;
import java.util.Comparator;

public class DegreeSequence
{

	/**
	 * Calculates the original degree sequence of WeightedGraph g
	 * 
	 * @return sorted degree Sequence of WeightedGraph g
	 */
	public static int[] calcDegreeSequence(final GraphInterface graph)
	{
		final int degreeSequence[] = new int[graph.getNumberOfVertices() + 1];
		for (int i = 0; i < (graph.getNumberOfVertices()); i++)
		{
			degreeSequence[i] = graph.getDegree(i);
		}
		DegreeSequence.sortSequenceByDegree(degreeSequence);
		return degreeSequence;
	}

	public static int getAnonymity(final int degSeq[])
	{

		int anonymity = Integer.MAX_VALUE;
		int gap = 0;
		for (int i = 1; i < degSeq.length; i += gap - 1)
		{
			gap = 0;
			for (int j = i; j < degSeq.length; j++)
			{
				gap++;
				if (degSeq[i] != degSeq[j])
				{
					gap = j - i;
					anonymity = Math.min(anonymity, gap);
					break;
				}
			}
			anonymity = Math.min(gap, anonymity);
		}
		return anonymity;

	}

	public static int getDelta(final int degSeq[])
	{

		int maxDegree = Integer.MIN_VALUE;
		for (int i = 1; i < degSeq.length; i++)
		{
			maxDegree = Math.max(maxDegree, degSeq[i]);
		}
		return maxDegree;
	}

	public static void sortSequenceByDegree(final int nodeSeq[])
	{
		Arrays.sort(nodeSeq);
	}

	/**
	 * 
	 * @param nodeSeq Sorted degree sequence by degree
	 */
	public static void sortSequenceByDegree(final Vertex nodeSeq[])
	{
		Arrays.sort(nodeSeq, 1, nodeSeq.length, new Comparator<Vertex>()
		{

			@Override
			public int compare(final Vertex o1, final Vertex o2)
			{
				return o1.getDegree() - o2.getDegree();
			}

		});
	}

	/**
	 * 
	 * @param nodeSeq Sorted degree sequence by node index
	 */
	public static void sortSequenceByIndex(final Vertex nodeSeq[])
	{
		Arrays.sort(nodeSeq, 1, nodeSeq.length, new Comparator<Vertex>()
		{

			@Override
			public int compare(final Vertex o1, final Vertex o2)
			{
				return o1.getPosition() - o2.getPosition();
			}
		});
	}

}
