package algorithm.dataStructures;

/**
 * A vertex object contains the vertex' degree, the vertex' original position and it's position after sorting a non-compact degree sequence by the
 * vertices' degree.
 * 
 * @author clemens
 * 
 */
public class Vertex implements Cloneable
{

	private int degree;

	private final int position;

	private int sortedPosition;

	/**
	 * 
	 * @param position The position of this vertex in a non-compact degree sequence.
	 * @param degree The degree of this vertex.
	 */
	public Vertex(final int position, final int degree)
	{
		this.degree = degree;
		this.position = position;
	}

	@Override
	public Vertex clone()
	{
		return new Vertex(this.position, this.degree);
	}

	/**
	 * 
	 * @return The degree of this vertex.
	 */
	public int getDegree()
	{
		return this.degree;
	}

	/**
	 * 
	 * @return The initial position of this vertex in its corresponding non-compact degree sequence.
	 */
	public int getPosition()
	{
		return this.position;
	}

	/**
	 * 
	 * @return The position of this vertex in its corresponding non-compact degree sequence after sorting the sequence by degrees.
	 */
	public int getSortedPosition()
	{
		return this.sortedPosition;
	}

	/**
	 * Update the degree of this vertex, e.g. after adding an edge which contains this vertex.
	 * 
	 * @param degree The degree of this vertex.
	 */
	public void setDegree(final int degree)
	{
		this.degree = degree;
	}

	/**
	 * The position of this degree in a non-compact degree sequence after sorting the sequence by degrees.
	 * 
	 * @param sortedPosition The new position of this vertex in a non-compact degree sequence.
	 */
	public void setSortedPosition(final int sortedPosition)
	{
		this.sortedPosition = sortedPosition;
	}

}
