package algorithm.graphRealizer;

import org.apache.commons.lang3.mutable.MutableInt;

import progOpt.TimeLogger;
import algorithm.UpperBound;

public class GraphRealizerConfig
{
	public static final MutableInt DIV_INNERITER_PARAM = new MutableInt(20000);
	public static final MutableInt DIV_INNERLOOPMOSTPROMISING_PARAM = new MutableInt(10000);
	public static final MutableInt DIV_MAXREORG_ADV_PARAM_VALUE = new MutableInt(100000);
	public static final MutableInt DIV_MAXREORG_SIMPLE_PARAM_VALUE = new MutableInt(1000000);
	public static final MutableInt DIV_OUTERITER_PARAM = new MutableInt(10000);
	public static final MutableInt MAX_INNERITER_PARAM = new MutableInt(10); // [2...infty]

	public static final MutableInt MAX_MAXREORG_PARAM_VALUE = new MutableInt(100); // [2....infty]
	public final static MutableInt MAX_OUTERITER_PARAM = new MutableInt(10); // [2...infty]

	public static int calcMaxIterations(final int innerIter, final int outerIter, final int innerLoopMostPromising)
	{
		return (innerIter * outerIter) + innerLoopMostPromising;
	}

	private long advReorgTimeStamp;

	protected int executedAdvancedReorganization = 0;

	protected int executedIterations = 0;
	protected int executedSimpleReorganizations = 0;
	final int innerIter;
	public final int innerLoopMostPromising;
	private long iterTimeStamp;
	public final int maxAdvReorg;

	public final int maxSimpleReorg;

	final int outerIter;

	private long simpleReorgTimeStamp;

	protected final String toStringSep = "; ";

	protected double totalAdvReorgTime = 0d;
	protected double totalIterTime = 0d;
	protected double totalSimpleReorgTime = 0;

	public GraphRealizerConfig(final int cost_Solution)
	{
		this(Math.min(GraphRealizerConfig.MAX_MAXREORG_PARAM_VALUE.intValue(),
				Math.max(1, GraphRealizerConfig.DIV_MAXREORG_SIMPLE_PARAM_VALUE.intValue() / Math.max(cost_Solution, 1))), Math.min(
				GraphRealizerConfig.MAX_MAXREORG_PARAM_VALUE.intValue(),
				Math.max(1, GraphRealizerConfig.DIV_MAXREORG_ADV_PARAM_VALUE.intValue() / Math.max(cost_Solution, 1))), Math.min(
				GraphRealizerConfig.MAX_INNERITER_PARAM.intValue(),
				Math.max(1, GraphRealizerConfig.DIV_INNERITER_PARAM.intValue() / Math.max(cost_Solution, 1))), Math.min(
				GraphRealizerConfig.MAX_OUTERITER_PARAM.intValue(),
				Math.max(1, GraphRealizerConfig.DIV_OUTERITER_PARAM.intValue() / Math.max(cost_Solution, 1))), Math.max(0,
				GraphRealizerConfig.DIV_INNERLOOPMOSTPROMISING_PARAM.intValue() / Math.max(cost_Solution, 1)), false);

		if (UpperBound.printRealizerParams == 2)
		{
			System.out.println("maxSimpleReorg:" + this.maxSimpleReorg + "\tmaxAdvReorg:" + this.maxAdvReorg + "\tinnerIter:" + this.innerIter
					+ "\touterIter:" + this.outerIter + "\tc:" + cost_Solution);
		}
	}

	public GraphRealizerConfig(final int maxSimpleReorg, final int maxAdvReorg, final int innerIter, final int outerIter,
			final int innerLoopMostPromising)
	{
		this(maxSimpleReorg, maxAdvReorg, innerIter, outerIter, innerLoopMostPromising, true);
	}

	private GraphRealizerConfig(final int maxSimpleReorg, final int maxAdvReorg, final int innerIter, final int outerIter,
			final int innerLoopMostPromising, final boolean log)
	{
		this.maxSimpleReorg = maxSimpleReorg;
		this.maxAdvReorg = maxAdvReorg;
		this.innerIter = innerIter;
		this.outerIter = outerIter;
		this.innerLoopMostPromising = innerLoopMostPromising;

		if (log && (UpperBound.printRealizerParams == 2))
		{
			System.out.println("maxSimpleReorg:" + this.maxSimpleReorg + "\tmaxAdvReorg:" + this.maxAdvReorg + "\tinnerIter:" + this.innerIter
					+ "\touterIter:" + this.outerIter + "\tinnerLoopMostPromising:" + this.innerLoopMostPromising);
		}
	}

	public double getAdvReorgTime()
	{
		return this.totalAdvReorgTime;
	}

	public double getAvgAdvReorgTime()
	{
		return this.executedAdvancedReorganization > 0 ? this.totalAdvReorgTime / this.executedAdvancedReorganization : 0;
	}

	public double getAvgSimpleReorgTime()
	{
		return this.executedSimpleReorganizations > 0 ? this.totalSimpleReorgTime / this.executedSimpleReorganizations : 0;
	}

	public int getMaxIter()
	{
		return GraphRealizerConfig.calcMaxIterations(this.innerIter, this.outerIter, this.innerLoopMostPromising);
	}

	public double getMaxReorgTime()
	{
		return this.totalSimpleReorgTime;
	}

	public int getNoExecutedAdvReorg()
	{
		return this.executedAdvancedReorganization;
	}

	public double getNoExecutedIterations()
	{
		return this.executedIterations;
	}

	public int getNoExecutedSimpleReorg()
	{
		return this.executedSimpleReorganizations;
	}

	public int getNoMaxReorgExec()
	{
		return this.executedSimpleReorganizations;
	}

	public double getSimpleReorgTime()
	{
		return this.totalSimpleReorgTime;
	}

	public double getTotalIterTime()
	{
		return this.totalIterTime;
	}

	public void increaseExecutedAdvancedReorganization()
	{
		this.advReorgTimeStamp = System.currentTimeMillis();
		this.executedAdvancedReorganization++;
	}

	public void increaseExecutedIterations()
	{
		this.iterTimeStamp = System.currentTimeMillis();
		this.executedIterations++;
	}

	public void increaseExecutedSimpleReorganizeSolution()
	{
		this.simpleReorgTimeStamp = System.currentTimeMillis();
		this.executedSimpleReorganizations++;

	}

	public void setExecutedAdvReorganizationTime()
	{
		this.totalAdvReorgTime += (System.currentTimeMillis() - this.advReorgTimeStamp) / 1000d;
	}

	public void setExecutedLastIterationTime()
	{
		this.totalIterTime += (System.currentTimeMillis() - this.iterTimeStamp) / 1000d;
	}

	public void setExecutedSimpleReorganizationTime()
	{
		this.totalSimpleReorgTime += (System.currentTimeMillis() - this.simpleReorgTimeStamp) / 1000d;
	}

	@Override
	public String toString()
	{// maxReorg,inner,out,innerLoopMostPromising
		final String s = "maxSimpleReorg:" + this.maxSimpleReorg + this.toStringSep + "maxAdvReorg:" + this.maxAdvReorg + this.toStringSep
				+ "innerIter=" + this.innerIter + this.toStringSep + "outerIter=" + this.outerIter + this.toStringSep + "innerLoopMostProm:"
				+ this.innerLoopMostPromising;
		return "GraphRealizerConfig [" + s + "] \t" + "[#execIterations=" + this.executedIterations + this.toStringSep + "totalTime="
				+ TimeLogger.timeFormatter.format(this.totalIterTime) + this.toStringSep + "avgSimpleReorg:"
				+ TimeLogger.timeFormatter.format(this.totalSimpleReorgTime / this.executedSimpleReorganizations) + " ("
				+ this.executedSimpleReorganizations + ")" + this.toStringSep + "avgAdvReorg:"
				+ TimeLogger.timeFormatter.format(this.totalAdvReorgTime / this.executedAdvancedReorganization) + " ("
				+ this.executedAdvancedReorganization + ")" + "]";
	}
}
