package algorithm.graphRealizer;

import org.apache.commons.lang3.mutable.MutableDouble;

import progOpt.TimeLogger;
import algorithm.dataStructures.Timer;

public class SophisticatedGraphRealizerConfigGenerator
{
	static class AdvConfig extends GraphRealizerConfig
	{

		private final double multFactor;
		private final double predictedTime;
		private final double supposedToTake;

		public AdvConfig(final int maxSimpleReorg, final int maxAdvReorg, final int innerIter, final int outerIter, final int innerLoopMostPromising,
				final double predictedTime, final double supposedToTake, final double multFactor)
		{
			super(maxSimpleReorg, maxAdvReorg, innerIter, outerIter, innerLoopMostPromising);
			this.predictedTime = predictedTime;
			this.supposedToTake = supposedToTake;
			this.multFactor = multFactor;
		}

		@Override
		public String toString()
		{// maxReorg,inner,out,innerLoopMostPromising
			final String s = "maxSimpleReorg:" + this.maxSimpleReorg + this.toStringSep + "maxAdvReorg:" + this.maxAdvReorg + this.toStringSep
					+ "innerIter=" + this.innerIter + this.toStringSep + "outerIter=" + this.outerIter + this.toStringSep + "innerLoopMostProm:"
					+ this.innerLoopMostPromising + this.toStringSep + "predictTime:" + TimeLogger.timeFormatter.format(this.predictedTime)
					+ this.toStringSep + "supposedToTake:" + TimeLogger.timeFormatter.format(this.supposedToTake) + this.toStringSep + "multFactor:"
					+ TimeLogger.timeFormatter.format(this.multFactor);
			return "Advanced GraphRealizerConfig \t[" + s + "] \n" + "                        \t[#execIterations=" + this.executedIterations
					+ this.toStringSep + "totalTime=" + TimeLogger.timeFormatter.format(this.totalIterTime) + this.toStringSep + "avgSimpleReorg:"
					+ TimeLogger.timeFormatter.format(this.totalSimpleReorgTime / this.executedSimpleReorganizations) + " ("
					+ this.executedSimpleReorganizations + ")" + this.toStringSep + "avgAdvReorg:"
					+ TimeLogger.timeFormatter.format(this.totalAdvReorgTime / this.executedAdvancedReorganization) + " ("
					+ this.executedAdvancedReorganization + ")" + "]\n";
		}
	}

	static class Config
	{
		public int innerIter;

		public int innerLoopMostPromising;

		public int maxAdvReorg;

		public int maxSimpleReorg;
		public int outerIter;

		public Config(final int maxSimpleReorg, final int maxAdvReorg, final int innerIter, final int outerIter, final int innerLoopMostPromising)
		{
			this.maxSimpleReorg = maxSimpleReorg;
			this.maxAdvReorg = maxAdvReorg;
			this.innerIter = innerIter;
			this.outerIter = outerIter;
			this.innerLoopMostPromising = innerLoopMostPromising;
		}

		public Config copy()
		{
			return new Config(this.maxSimpleReorg, this.maxAdvReorg, this.innerIter, this.outerIter, this.innerLoopMostPromising);
		}

		@Override
		public String toString()
		{
			return "Config [maxSimpleReorg=" + this.maxSimpleReorg + ", maxAdvReorg=" + this.maxAdvReorg + ", innerIter=" + this.innerIter
					+ ", outerIter=" + this.outerIter + ", innerLoopMostPromising=" + this.innerLoopMostPromising + "]";
		}

	}

	static class Statistic
	{

		private static double update(final double value1, final double value2, final boolean valuableValue)
		{
			if (valuableValue)
				return ((1d - SophisticatedGraphRealizerConfigGenerator.SOPHISTICATED_CONFIG_GENERATOR_UPDATE_VALUABLE_STATISTIC.doubleValue()) * value1)
						+ (SophisticatedGraphRealizerConfigGenerator.SOPHISTICATED_CONFIG_GENERATOR_UPDATE_VALUABLE_STATISTIC.doubleValue() * value2);
			else
				return ((1d - SophisticatedGraphRealizerConfigGenerator.SOPHISTICATED_CONFIG_GENERATOR_UPDATE_NONE_VALUABLE_STATISTIC.doubleValue()) * value1)
						+ (SophisticatedGraphRealizerConfigGenerator.SOPHISTICATED_CONFIG_GENERATOR_UPDATE_NONE_VALUABLE_STATISTIC.doubleValue() * value2);
		}

		private double avg_AdvReorgTimePerIter;
		private double avg_SimpleReorgTimePerIter;
		private double avgAdvReorgTimePerApplication;
		private double avgIterTime;

		private double avgSimpleReorgTimePerApplication;
		private int lastCostValue;

		private double multFactor = 1d;

		Statistic(final GraphRealizerConfig lastConfig, final int cost)
		{
			this.avgIterTime = lastConfig.getTotalIterTime() / lastConfig.getNoExecutedIterations();
			this.avg_AdvReorgTimePerIter = lastConfig.getAdvReorgTime() / lastConfig.getNoExecutedIterations();
			this.avg_SimpleReorgTimePerIter = lastConfig.getSimpleReorgTime() / lastConfig.getNoExecutedIterations();
			this.avgSimpleReorgTimePerApplication = lastConfig.getAvgSimpleReorgTime();
			this.avgAdvReorgTimePerApplication = lastConfig.getAvgAdvReorgTime();
			this.lastCostValue = cost;
		}

		public double getAvg_AdvReorgTimePerIter()
		{
			return this.avg_AdvReorgTimePerIter;
		}

		public double getAvg_SimpleReorgTimePerIter()
		{
			return this.avg_SimpleReorgTimePerIter;
		}

		public double getAvgAdvReorgTimePerApplication()
		{
			return this.avgAdvReorgTimePerApplication;
		}

		public double getAvgIterTime()
		{
			return this.avgIterTime;
		}

		public double getAvgSimpleReorgTimePerApplication()
		{
			return this.avgSimpleReorgTimePerApplication;
		}

		public void updateStatistic(final GraphRealizerConfig lastConfig, final int cost)
		{

			this.multFactor = ((double) this.lastCostValue) / ((double) cost);
			this.multFactor = Math.max(1 - (SophisticatedGraphRealizerConfigGenerator.maxBound_multFactor.doubleValue() - 1),
					Math.min(SophisticatedGraphRealizerConfigGenerator.maxBound_multFactor.doubleValue(), this.multFactor));
			this.lastCostValue = cost;

			if (this.multFactor > 1d)
			{
				this.multFactor = (SophisticatedGraphRealizerConfigGenerator.MULTFACTOR_INCREASE_COST.doubleValue() * SophisticatedGraphRealizerConfigGenerator.maxBound_multFactor
						.doubleValue()) + ((1d - SophisticatedGraphRealizerConfigGenerator.MULTFACTOR_INCREASE_COST.doubleValue()) * this.multFactor);
			} else if ((this.multFactor <= 1d) && (lastConfig instanceof AdvConfig))
			{
				final AdvConfig advConfig = (AdvConfig) lastConfig;
				this.multFactor = (SophisticatedGraphRealizerConfigGenerator.MULTFACTOR_DECREASE_COST.doubleValue() * this.multFactor)
						+ ((1d - SophisticatedGraphRealizerConfigGenerator.MULTFACTOR_DECREASE_COST.doubleValue()) * Math.max(
								1 - (SophisticatedGraphRealizerConfigGenerator.maxBound_multFactor.doubleValue() - 1), Math.min(
										(advConfig.getTotalIterTime() / advConfig.predictedTime),
										SophisticatedGraphRealizerConfigGenerator.maxBound_multFactor.doubleValue())));
			}

			this.avgIterTime = Statistic.update(this.avgIterTime, lastConfig.getTotalIterTime() / lastConfig.getNoExecutedIterations(), true);
			this.avg_AdvReorgTimePerIter = Statistic.update(this.avg_AdvReorgTimePerIter,
					lastConfig.getAdvReorgTime() / lastConfig.getNoExecutedIterations(), lastConfig.getNoExecutedAdvReorg() > 0);
			this.avg_SimpleReorgTimePerIter = Statistic.update(this.avg_SimpleReorgTimePerIter,
					lastConfig.getSimpleReorgTime() / lastConfig.getNoExecutedIterations(), lastConfig.getNoExecutedSimpleReorg() > 0);
			this.avgSimpleReorgTimePerApplication = Statistic.update(this.avgSimpleReorgTimePerApplication, lastConfig.getAvgSimpleReorgTime(),
					lastConfig.getNoExecutedSimpleReorg() > 0);
			this.avgAdvReorgTimePerApplication = Statistic.update(this.avgAdvReorgTimePerApplication, lastConfig.getAvgAdvReorgTime(),
					lastConfig.getNoExecutedAdvReorg() > 0);
		}
	}

	public static final MutableDouble Config_Generator_HighQuality_Improve_Fac = new MutableDouble(7);
	public final static MutableDouble InnerIterTOOuterIter_Frac = new MutableDouble(10f);

	public static final MutableDouble maxBound_multFactor = new MutableDouble(1.25d);

	public final static MutableDouble maxReorgTOInnerIter_Frac = new MutableDouble(1.8f);

	public final static MutableDouble maxSimpleReorgTOmaxAdvReorg_Frac = new MutableDouble(2f);

	public static final MutableDouble MULTFACTOR_DECREASE_COST = new MutableDouble(0.6d);

	public static final MutableDouble MULTFACTOR_INCREASE_COST = new MutableDouble(0.8d);

	public static MutableDouble SOPHISTICATED_CONFIG_GENERATOR_UPDATE_NONE_VALUABLE_STATISTIC = new MutableDouble(0.2d);// [0,1]

	public static MutableDouble SOPHISTICATED_CONFIG_GENERATOR_UPDATE_VALUABLE_STATISTIC = new MutableDouble(0.75d);// [0,1]

	public static final MutableDouble timeThresholdForIncrease = new MutableDouble(1d);

	public static final MutableDouble timeThresholdForIncreaseRandom = new MutableDouble(0.5);

	/**
	 * params={maxReorg,inner,out,innerLoopMostPromising}
	 * 
	 * @param params
	 * @param increase
	 * @param lastConfig
	 */
	private static void balanceValues(final Config params)
	{
		params.innerIter = Math.min(GraphRealizerConfig.MAX_INNERITER_PARAM.intValue(),
				Math.max(1, Math.round(params.maxSimpleReorg / SophisticatedGraphRealizerConfigGenerator.maxReorgTOInnerIter_Frac.floatValue())));
		params.outerIter = Math.min(GraphRealizerConfig.MAX_OUTERITER_PARAM.intValue(),
				Math.max(1, Math.round(params.innerIter / SophisticatedGraphRealizerConfigGenerator.InnerIterTOOuterIter_Frac.floatValue())));
	}

	private static double estimateRuntime(final Config params, final Statistic statistics)
	{
		final double lastRunTimePerIter = statistics.getAvgIterTime();
		final int numberOfIter = GraphRealizerConfig.calcMaxIterations(params.innerIter, params.outerIter, params.innerLoopMostPromising);

		final double costReorg = (params.maxAdvReorg * statistics.getAvgAdvReorgTimePerApplication())
				+ (params.maxSimpleReorg * statistics.getAvgSimpleReorgTimePerApplication());
		final double overheadPerIter = costReorg / (statistics.getAvg_AdvReorgTimePerIter() + statistics.getAvg_SimpleReorgTimePerIter());

		final double costPerIter = (lastRunTimePerIter * numberOfIter) * overheadPerIter;
		// System.out.println("estimated time:" + TimeLogger.timeFormatter.format(costPerIter) + "s for \t" + params);
		return Math.max(costPerIter, costReorg) * statistics.multFactor;
	}

	/**
	 * Calculates new config params for {@code GraphRealizer} based on a prediction model build upon the last run.
	 * 
	 * @param totalIterTime
	 * @param maxTimeAllowedToTake max amount of time the new config is allowed to take
	 * @return
	 */
	public static Config getNewParams(final double maxTimeAllowedToTake, final Statistic statistics)
	{
		// params={maxReorg,inner,out,innerLoopMostPromising}
		// final int[] params = { 1, 1, 1, 0 };
		final Config params = new Config(1, 0, 1, 1, 0);

		Config lastParams = params.copy();
		while (SophisticatedGraphRealizerConfigGenerator.estimateRuntime(params, statistics) < maxTimeAllowedToTake)
		{
			lastParams = params.copy();

			// all other parameters are related by constant fraction to maxSimpleReorg
			params.maxSimpleReorg++;

			{// balance maxSimpleReorg and maxAdvReorg
				int increase = Math.round(params.maxSimpleReorg
						/ SophisticatedGraphRealizerConfigGenerator.maxSimpleReorgTOmaxAdvReorg_Frac.floatValue())
						- params.maxAdvReorg;
				if (params.maxAdvReorg == 0)
				{
					increase = 1;
				}
				final int orgValue = params.maxAdvReorg;
				params.maxAdvReorg += increase;
				while ((SophisticatedGraphRealizerConfigGenerator.estimateRuntime(params, statistics) > maxTimeAllowedToTake)
						&& (params.maxAdvReorg > orgValue))
				{
					params.maxAdvReorg--;
				}
			}
			{// adjust iter params
				if (params.innerLoopMostPromising < GraphRealizer.MAX_INNERLOOP_ADD_MOST_PROMISING.intValue())
				{
					params.innerLoopMostPromising++;
				} else
				{
					params.innerLoopMostPromising = 0;
					SophisticatedGraphRealizerConfigGenerator.balanceValues(params);
				}
			}
		}
		while ((SophisticatedGraphRealizerConfigGenerator.estimateRuntime(lastParams, statistics) < maxTimeAllowedToTake)
				&& (lastParams.maxSimpleReorg < (params.maxSimpleReorg << 1)))
		{
			lastParams.maxSimpleReorg++;
		}
		lastParams.maxSimpleReorg = Math.max(1, lastParams.maxSimpleReorg - 1);

		return Math.abs(SophisticatedGraphRealizerConfigGenerator.estimateRuntime(params, statistics) - maxTimeAllowedToTake) < Math
				.abs(SophisticatedGraphRealizerConfigGenerator.estimateRuntime(lastParams, statistics) - maxTimeAllowedToTake) ? params : lastParams;
	}

	private boolean highQuality = false;

	private GraphRealizerConfig lastConfig;

	private double nextTimeCache = -1d;

	private final int solutionDiffToOrig;

	private Statistic statistics;

	public SophisticatedGraphRealizerConfigGenerator(final int solutionDiffToOrig)
	{
		this.solutionDiffToOrig = solutionDiffToOrig;
	}

	public void disableHighQuality()
	{
		this.highQuality = false;
	}

	public void enableHighQualityMode()
	{
		this.highQuality = true;
	}

	public GraphRealizerConfig getNextConfig(final int cost)
	{

		if (this.lastConfig == null)
		{// default
			final double fac = this.highQuality ? SophisticatedGraphRealizerConfigGenerator.Config_Generator_HighQuality_Improve_Fac.doubleValue()
					: 1d;
			final int costSolution = cost + this.solutionDiffToOrig;
			this.lastConfig = new GraphRealizerConfig(costSolution / (int) Math.round(fac));

		} else
		{
			if (this.statistics == null)
			{
				this.statistics = new Statistic(this.lastConfig, cost);
			} else
			{
				this.statistics.updateStatistic(this.lastConfig, cost);
			}

			final double maxTimeAllowedToTake = this.nextTimeCache == -1d ? this.setTimeForNextConfig(null) : this.nextTimeCache;
			this.nextTimeCache = -1d;// mark time value as read
			final Config params = SophisticatedGraphRealizerConfigGenerator.getNewParams(maxTimeAllowedToTake, this.statistics);

			this.lastConfig = new AdvConfig(params.maxSimpleReorg, params.maxAdvReorg, params.innerIter, params.outerIter,
					params.innerLoopMostPromising, SophisticatedGraphRealizerConfigGenerator.estimateRuntime(params, this.statistics),
					maxTimeAllowedToTake, this.statistics.multFactor);
		}

		return this.lastConfig;
	}

	public double setTimeForNextConfig(final Timer timer)
	{
		this.nextTimeCache = SophisticatedGraphRealizerConfigGenerator.timeThresholdForIncrease.floatValue()
				+ (SophisticatedGraphRealizerConfigGenerator.timeThresholdForIncrease.floatValue()
						* SophisticatedGraphRealizerConfigGenerator.timeThresholdForIncreaseRandom.doubleValue() * Math.random());
		this.nextTimeCache *= this.highQuality ? SophisticatedGraphRealizerConfigGenerator.Config_Generator_HighQuality_Improve_Fac.doubleValue()
				: 1d;
		if ((timer != null) && timer.isMaxTimeSet())
		{
			this.nextTimeCache = Math.min(this.nextTimeCache,
					Math.max(1, GraphRealizer.Safety_Buffer_Fac_MAX_Time_Configer.doubleValue() * timer.getRemainingTime()));
		}
		return this.nextTimeCache;
	}
}
