package benchmarks;

import java.io.IOException;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Arrays;
import java.util.Collections;
import java.util.Locale;
import java.util.Vector;

import dataStructures.Store;
import dataStructures.utils.ArrayS;

public class LogStringBuilder
{
	private class Tmp extends Store<Integer, String> implements Comparable<Tmp>
	{
		public Tmp(final int i, final String k)
		{
			super(i, k);
		}

		@Override
		public int compareTo(final Tmp o)
		{
			return this.getS2().compareTo(o.getS2());
		}

		@Override
		public boolean equals(final Object obj)
		{
			if (!(obj instanceof Tmp))
				return false;
			return this.getS2().equalsIgnoreCase(((Tmp) obj).getS2());
		}

		@Override
		public int hashCode()
		{
			return this.getS2().hashCode();
		}

	}

	public static String[] arrKeys = { "file", "n",
			"m",
			"maxDeg",
//			"#emptyBlocks","#blocksOfSize1-k",
			"k",
			"timeToLoadGraph",
//			"oldDA", "oldDATime","quickDA", "quickDATime",
			LogStringBuilder.LogKey_KOLJA_HEURISTIC, 
			LogStringBuilder.LogKey_KOLJA_HEURISTIC_TIME, 
			LogStringBuilder.LogKey_FAST_UPPERBOUND_HEURISTIC,
			LogStringBuilder.LogKey_FAST_UPPERBOUND_HEURISTIC_TIME,
			LogStringBuilder.LogKey_SEMI_SOPHISTICATED_UPPERBOUND_HEURISTIC, 
			LogStringBuilder.LogKey_SEMI_SOPHISTICATED_UPPERBOUND_HEURISTIC_TIME, 
			LogStringBuilder.LogKey_SOPHISTICATED_UPPERBOUND_HEURISTIC, 
			LogStringBuilder.LogKey_SOPHISTICATED_UPPERBOUND_HEURISTIC_TIME, 
			"numberOfSolutions", "LastStoredSolInCacheAtCost", "solutionSize",
			"preprocessCost", "hopeforSuccussRate", "totalgetNoOfAEGHopeTests", "instancesConsidered", "realizable",
			TestAnonymizerCaller.LogKey_TEST_SOL_FOR_REALIZABLE, "timeToIniDegreeAnonymizer",
			LogStringBuilder.LogKey_TIME_FOR_GET_ALL_K_ANON_REAL_SEQUENCES, "timeToComputeDegreeSequence", "timeForGraphRealizing",
			"lastExaminedCost", "lastUpperBoundInSolutionIterator",
	// "minDepth", "#vertices1", "ratio1", "minDepthAdvanced",
	// "#vertices2", "ratio2", "maxDegPreprocessed", "initSequence",
	// "hopeDestroyedAtDepth", "hopeDestroyedAtDepthAdvanced",
	};

	private static LogStringBuilder log;

	public static final String LogKey_FAST_UPPERBOUND_HEURISTIC = "fastUpperboundHeuristic";
	public static final String LogKey_FAST_UPPERBOUND_HEURISTIC_TIME = "fastUpperboundHeuristicTime";

	public static final String LogKey_KOLJA_HEURISTIC = "KoljaHeuristic";
	public static final String LogKey_KOLJA_HEURISTIC_TIME = "KoljaHeuristicTime";

	// "%-25s", "%-25s", "%-25s", "%-25s", "%-25s", "%-25s", "%-25s", "%-25s",
	// "%-25s", "%-25s", "%-25s", "%-25s", "%-25s", "%-25s",
	// "%-25s", "%-25s", "%-25s", "%-25s", "%-25s" };

	public static final String LogKey_SOPHISTICATED_UPPERBOUND_HEURISTIC = "sophUpperboundHeuristic";
	public static final String LogKey_SOPHISTICATED_UPPERBOUND_HEURISTIC_TIME = "sophUpperboundHeuristicTime";

	public static final String LogKey_SEMI_SOPHISTICATED_UPPERBOUND_HEURISTIC = "semiSophUpperboundHeuristic";
	public static final String LogKey_SEMI_SOPHISTICATED_UPPERBOUND_HEURISTIC_TIME = "semiSophUpperboundHeuristicTime";

	public static final String LogKey_TIME_FOR_GET_ALL_K_ANON_REAL_SEQUENCES = "timeToGetAllKAnonRealSequences";

	public static LogStringBuilder getLogger()
	{
		if (LogStringBuilder.log == null)
		{
			LogStringBuilder.log = new LogStringBuilder();
		}
		return LogStringBuilder.log;
	}

	String[] arrValues = new String[LogStringBuilder.arrKeys.length];
	public final String[] formatStrings;// = { "%-30s", "%-25s", "%-25s",
										// "%-25s", "%-25s", "%-25s", "%-25s",
										// "%-25s", "%-25s", "%-25s",

	private final DecimalFormat myFormatter;

	// public static final String LogKey_UPPERBOUND_HEURISTIC = "#upperboundHeuristic";

	private final String sep;

	private long timeStamp;

	private final Vector<Tmp> vec;

	private LogStringBuilder()
	{
		this(";");
	}

	private LogStringBuilder(final String sep)
	{
		this.sep = sep;
		this.formatStrings = new String[LogStringBuilder.arrKeys.length];
		{
			for (int l = 0; l < LogStringBuilder.arrKeys.length; l++)
			{
				this.formatStrings[l] = "%-" + Math.max(15, LogStringBuilder.arrKeys[l].length() + 4) + "s";
			}
			this.formatStrings[0] = "%-30s";
		}
		this.vec = new Vector<>();
		int i = 0;
		for (final String k : LogStringBuilder.arrKeys)
		{
			this.vec.add(new Tmp(i++, k));
		}

		Collections.sort(this.vec);

		final Locale locale = new Locale("en", "US");

		this.myFormatter = (DecimalFormat) NumberFormat.getNumberInstance(locale);
		this.myFormatter.applyPattern("###,###,###.###");
	}

	public void addEntry(final String name, final float f)
	{
		this.addEntry(name, this.myFormatter.format(f));
	}

	public void addEntry(final String name, final int value)
	{
		this.addEntry(name, this.myFormatter.format(value));
	}

	public void addEntry(final String string, final int value, final boolean printTime)
	{
		this.addEntry(string, value);
		if (printTime)
		{
			this.addEntry(string+"Time", this.myFormatter.format((System.currentTimeMillis() - this.timeStamp) / 1000d));
			this.startTime();
		}
	}

	public void addEntry(final String name, final String value)
	{
		final int i = Collections.binarySearch(this.vec, new Tmp(0, name));
		if (i < 0)
			throw new IllegalAccessError(name);
		else
		{
			this.arrValues[this.vec.get(i).getS1()] = value;
		}
	}

	public String[] addSuffic(final String[] arr)
	{
		final String[] tmpKeys = Arrays.copyOf(arr, arr.length);
		for (int l = 0; l < tmpKeys.length; l++)
		{
			tmpKeys[l] = (tmpKeys[l] == null ? "" : tmpKeys[l]) + this.sep;
		}
		return tmpKeys;
	}

	public String getEntry(final String string)
	{
		final int i = Collections.binarySearch(this.vec, new Tmp(0, string));
		if (i < 0)
			throw new IllegalAccessError();
		else
		{
			return this.arrValues[this.vec.get(i).getS1()];
		}
	}
	
	public void addTimeEntry(final String string)
	{
		this.addTimeEntry(string, true);
	}

	public void addTimeEntry(final String string,boolean restartTimeCounter)
	{
		this.addEntry(string, this.myFormatter.format((System.currentTimeMillis() - this.timeStamp) / 1000d));
		if (restartTimeCounter)
			this.startTime();
	}
	
	public void cleanValues()
	{
		Arrays.fill(this.arrValues, null);
	}

	public void printHeader(final OutputStream outStream) throws IOException
	{
		final String s = ArrayS.toString(Arrays.asList(this.formatStrings), "");
		final String[] tmpKeys = this.addSuffic(LogStringBuilder.arrKeys);
		outStream.write(String.format(s, (Object[]) tmpKeys).getBytes());
		// final StringBuffer out = new
		// StringBuffer(ArrayS.toString(Arrays.asList(LogStringBuilder.arrKeys),
		// this.sep));
		outStream.write("\n".getBytes());
		// outStream.write(out.toString().getBytes());
		outStream.flush();
	}

	public void printOutput(final OutputStream outStream) throws IOException
	{
		final String s = ArrayS.toString(Arrays.asList(this.formatStrings), "");
		final Vector<String> vecT = new Vector<>();
		for (final String key : LogStringBuilder.arrKeys)
		{
			final int binarySearch = Collections.binarySearch(this.vec, new Tmp(0, key));
			vecT.add(this.arrValues[this.vec.get(binarySearch).getS1()]);
		}
		final String[] addSuffic = this.addSuffic(vecT.toArray(new String[] {}));
		outStream.write(String.format(s, (Object[]) addSuffic).getBytes());
		outStream.write("\n".getBytes());
		outStream.flush();
	}

	public void startTime()
	{
		this.timeStamp = System.currentTimeMillis();
	}
}
