package benchmarks;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import reader.GraphReader;
import algorithm.DegreeAnonymizer;
import algorithm.SolutionDegreeIterator;
import algorithm.dataStructures.GraphInterface;
import algorithm.graphRealizer.GraphRealizer;

public class LowerBoundAllStepsReported {

	public static void main(final String[] args) throws FileNotFoundException, IOException
	{
		ClassLoader.getSystemClassLoader().setDefaultAssertionStatus(false);

		MainCLIAnonymizerAndAlternativeRealizer.logLevel = 0;
		SolutionDegreeIterator.printLogForEachCostValue = true;
		DegreeAnonymizer.logDR = 0;
		GraphRealizer.log = 0;

		int timelimit = 60; // timelimit for one instance in minutes
		String[] graphs = { "graphs/10thDimacs/coAuthorsCiteseer.graph"};
		int[] kValues = { 2, 3, 4, 5, 7, 9, 10, 15, 20, 30, 50, 100, 200};

		if (args.length >= 1)
		{
			graphs = new String[]{ args[0] };
		}
		
		{// write header
			LogStringBuilder.getLogger().printHeader(System.out);

			FileOutputStream outStream = new FileOutputStream(new File("singleStep.log"), true);
			LogStringBuilder.getLogger().printHeader(outStream);
			outStream = new FileOutputStream(new File("lowerBound_singleStep.log"), true);
			LogStringBuilder.getLogger().printHeader(outStream);
			outStream.close();
		}

		for (final String fileName : graphs)
		{
		
			LogStringBuilder.getLogger().startTime();
			final GraphInterface graph = new GraphReader(fileName).readGraphFile();
			LogStringBuilder.getLogger().addTimeEntry("timeToLoadGraph");
			String timeToLoadGraph = LogStringBuilder.getLogger().getEntry("timeToLoadGraph");
					
			for (final int k : kValues)
			{
				MainCLIAnonymizerAndAlternativeRealizer.timeStamp = System.currentTimeMillis();

				LogStringBuilder.getLogger().cleanValues();
				LogStringBuilder.getLogger().startTime();

				LogStringBuilder.getLogger().addEntry("timeToLoadGraph", timeToLoadGraph);

				LogStringBuilder.getLogger().addEntry("file", fileName);
				LogStringBuilder.getLogger().addEntry("n", graph.getNumberOfVertices());
				LogStringBuilder.getLogger().addEntry("m", graph.getNumberOfEdges());
				LogStringBuilder.getLogger().addEntry("maxDeg", graph.getDelta());

				LogStringBuilder.getLogger().addEntry("k", k);

				final TestAnonymizerCaller s = new TestAnonymizerCaller(graph, k, true);
				s.start();
				try
				{
					s.join(1000 * 60 * timelimit);

				} catch (final InterruptedException e)
				{

				}
				s.writeLogging();

				if (s.isAlive())
				{
					s.interrupt();
					s.stop();
				}

				{// printOutput
					LogStringBuilder.getLogger().printOutput(System.out);

					final FileOutputStream outStream = new FileOutputStream(new File("lowerBound_singleStep.log"), true);
					LogStringBuilder.getLogger().printOutput(outStream);
					outStream.close();

					LogStringBuilder.getLogger().cleanValues();
				}
				System.gc();
			}
		}
	}

	
}
