package benchmarks;

import gnu.trove.iterator.TIntIterator;
import gnu.trove.set.TIntSet;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ExecutionException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

import reader.GraphReader;
import algorithm.DegreeAnonymizer;
import algorithm.UpperBound;
import algorithm.alternativeRealizer.GreedyGraphConstructor;
import algorithm.dataStructures.CompactDegreeSequence;
import algorithm.dataStructures.DegreeSequence;
import algorithm.dataStructures.Edge;
import algorithm.dataStructures.GraphInterface;

public class MainCLIAnonymizerAndAlternativeRealizer
{

	public static int cost;

	public static String instance = null;

	public static int k;

	public static int logLevel = 0;

	public static final int maxTimeInMin = 30;

	public static String out = null;

	public static boolean simulate = true;
	public static long timeStamp = 0;

	public static void main(final String[] args) throws InterruptedException, ExecutionException
	{
		new MainCLIAnonymizerAndAlternativeRealizer().calcSolutionSet(args);

	}

	public static int sum(final int[] remainingCost)
	{
		int s = 0;
		for (int i = 0; i < remainingCost.length; i++)
		{
			s += remainingCost[i];
		}
		return s;
	}

	public Set<Map<Integer, TIntSet>> calcSolutionSet(final String args[])
	{
		MainCLIAnonymizerAndAlternativeRealizer.timeStamp = System.currentTimeMillis();
		final long startintTime = System.currentTimeMillis();
		final Options options = new Options();
		final Option help = new Option("help", "Print this help");
		OptionBuilder.withArgName("k");
		OptionBuilder.hasArg();
		OptionBuilder.withDescription("Specify k (anonymization level)");
		final Option kOption = OptionBuilder.create("k");
		final Option serializeOption = OptionBuilder.create("serialize");

		final Option simulateOption = OptionBuilder.create("simulate");
		final Option deserializeOption = OptionBuilder.create("deserialize");
		final Option enabledataReductionRules = new Option("drr", "data-reduction-rules", false, "Enable data reduction rules");
		final Option disabledataReductionRules = new Option("nodrr", "disable-data-reduction-rules", false, "Disable data reduction rules (Default)");
		final Option verbose = new Option("verbose", "Generate more output.");
		OptionBuilder.withArgName("selection-strategy");
		OptionBuilder.withLongOpt("selection-strategy");
		OptionBuilder.hasArg();
		OptionBuilder.withDescription("specify vertex selection strategy for graph construction (1 = Minimum common neighbors, "
				+ "2 = minimum sum of neighbor degrees, " + "3 = random (200 iteration), 4 = Strategy 1 Refined)");
		final Option selectionStrategy = OptionBuilder.create("ss");
		OptionBuilder.withArgName("processing-strategy");
		OptionBuilder.withLongOpt("processing-strategy");
		OptionBuilder.hasArg();
		OptionBuilder.withDescription("specify vertex processing strategy for graph construction  (1 = vertices with largest difference first, "
				+ "2 = vertices with fewest common neighbors in set first, 3 = random (500 iterations), 4 = Strategy 1 Refined)");

		final Option processingStrategy = OptionBuilder.create("ps");

		OptionBuilder.withArgName("ss-passes");
		OptionBuilder.withLongOpt("ss-passes");
		OptionBuilder.hasArg();
		OptionBuilder.withDescription("specify passes for random vertex selection within graph construction");
		final Option selection_random_passes = OptionBuilder.create("ss_passes");

		OptionBuilder.withArgName("ps-passes");
		OptionBuilder.withLongOpt("ps-passes");
		OptionBuilder.hasArg();
		OptionBuilder.withDescription("specify passes for random vertex processing within graph construction");
		final Option processing_random_passes = OptionBuilder.create("ps_passes");

		OptionBuilder.withArgName("if");
		OptionBuilder.withLongOpt("input-file");
		OptionBuilder.hasArg();
		OptionBuilder.withDescription("specify input file");
		final Option inputFile = OptionBuilder.create("if");

		OptionBuilder.withArgName("out");
		OptionBuilder.withLongOpt("output-directory");
		OptionBuilder.hasArg();
		OptionBuilder.withDescription("specify directory for solution set lists");
		final Option outputDirectory = OptionBuilder.create("out");

		options.addOption(outputDirectory);
		options.addOption(serializeOption);
		options.addOption(simulateOption);
		options.addOption(deserializeOption);
		options.addOption(help);
		options.addOption(kOption);
		options.addOption(inputFile);
		options.addOption(enabledataReductionRules);
		options.addOption(disabledataReductionRules);
		options.addOption(processingStrategy);
		options.addOption(selectionStrategy);
		options.addOption(verbose);
		options.addOption(selection_random_passes);
		options.addOption(processing_random_passes);

		final CommandLineParser parser = new GnuParser();
		CommandLine cmd = null;

		try
		{
			cmd = parser.parse(options, args);
		} catch (final ParseException e)
		{
			e.printStackTrace();
		}

		if (cmd.hasOption("help"))
		{
			final HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp("kanonymizer", options);
			System.exit(0);
		}

		final boolean datareduction;

		GraphReader reader = null;
		if (cmd.hasOption("input-file"))
		{
			System.out.println("#Input file : " + cmd.getOptionValue("input-file"));
			if (!new File(cmd.getOptionValue("input-file")).exists())
			{
				System.out.println("Input file does not exist. Terminating.");
				System.exit(1);
			}
			reader = new GraphReader(cmd.getOptionValue("input-file"));
			final String splits[] = cmd.getOptionValue("input-file").split(System.getProperty("file.separator"));
			if (splits.length > 1)
			{
				MainCLIAnonymizerAndAlternativeRealizer.instance = splits[splits.length - 1];
			} else
			{
				MainCLIAnonymizerAndAlternativeRealizer.instance = splits[0];
			}
		} else
		{
			final HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp("kanonymizer", options);
			System.out.println("Input file not specified. Terminating.");
			System.exit(1);
		}

		if (cmd.hasOption("output-directory"))
		{
			System.out.println("#Output Directory : " + cmd.getOptionValue("output-directory"));
			if (!new File(cmd.getOptionValue("output-directory")).mkdirs() && !new File(cmd.getOptionValue("output-directory")).exists())
			{
				System.out.println("Can not create Output Directory!");
				System.exit(1);
			}
			MainCLIAnonymizerAndAlternativeRealizer.out = cmd.getOptionValue("output-directory");
		}

		if (cmd.hasOption("k"))
		{
			try
			{
				MainCLIAnonymizerAndAlternativeRealizer.k = Integer.parseInt(cmd.getOptionValue("k"));
				System.out.println("#k: " + MainCLIAnonymizerAndAlternativeRealizer.k);
			} catch (final NumberFormatException ie)
			{
				System.out.println("k must be specified as a positive Integer. Terminating.");
				System.exit(1);
			}
		} else
		{
			final HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp("kanonymizer", options);
			System.out.println("k not specified. Terminating.");
			System.exit(1);
		}
		if (cmd.hasOption("drr"))
		{
			datareduction = true;
			System.out.println("#Data reduction rule enabled");
		} else
		{
			datareduction = false;
		}

		final List<Integer> selectionStrategies = new LinkedList<>();

		if (cmd.hasOption("ss"))
		{
			try
			{
				final int ss = Integer.parseInt(cmd.getOptionValue("ss"));
				if ((ss < 1) || (ss > 4))
					throw new NumberFormatException();
				selectionStrategies.add(ss);
			} catch (final NumberFormatException ie)
			{
				System.out.println("Selection mode must be specified as an integer [0-3]. Terminating.");
				System.exit(1);
			}

		} else
		{
			selectionStrategies.add(GreedyGraphConstructor.SELECTION_STRATEGY_LEAST_COMMON_NEIGHBORS); // Selection
			// Strategy
			// 1
			selectionStrategies.add(GreedyGraphConstructor.SELECTION_STRATEGY_LEAST_COMMON_NEIGHBORS_REFINED); // Selection
			// Strategy 2
			selectionStrategies.add(GreedyGraphConstructor.SELECTION_STRATEGY_LEAST_NEIGHBOR_DEGREE_SUM); // improved
			// Selection
			// Strategy 1
			selectionStrategies.add(GreedyGraphConstructor.SELECTION_STRATEGY_RANDOM); // Selection
			// Strategy
			// 3
		}

		final List<Integer> processingStrategies = new LinkedList<>();
		if (cmd.hasOption("ps"))
		{
			try
			{
				final int ps = Integer.parseInt(cmd.getOptionValue("ps"));
				if ((ps < 1) || (ps > 4))
					throw new NumberFormatException();
				processingStrategies.add(ps);
			} catch (final NumberFormatException ie)
			{
				System.out.println("Ordering mode must be specified as an integer [0-3]. Terminating.");
				System.exit(1);
			}
		}

		else
		{
			processingStrategies.add(GreedyGraphConstructor.PROCESSING_STRATEGY_MAXDIFF_FIRST); // Processing
			// Strategy 1
			// orderingMethods.add(processingStrategy2); // Processing Strategy
			// 2
			// orderingMethods.add(processingStrategy3); // Processing Strategy
			// 3
			processingStrategies.add(GreedyGraphConstructor.PROCESSING_STRATEGY_MAXDIFF_REFINED); // Processing
			// Strategy
			// 1 Refined
		}

		if (cmd.hasOption("ss_passes"))
		{
			GreedyGraphConstructor.RANDOM_SELECTION_PASSES = Integer.parseInt(cmd.getOptionValue("ss_passes"));
			System.out.println("Setting passes for vertex selection to " + GreedyGraphConstructor.RANDOM_SELECTION_PASSES);
		}

		if (cmd.hasOption("ps_passes"))
		{
			GreedyGraphConstructor.RANDOM_PROCESSING_PASSES = Integer.parseInt(cmd.getOptionValue("ps_passes"));
			System.out.println("Setting passes for vertex processing to " + GreedyGraphConstructor.RANDOM_PROCESSING_PASSES);
		}

		if (cmd.hasOption("verbose"))
		{
			MainCLIAnonymizerAndAlternativeRealizer.logLevel = 3;
		} else
		{
			MainCLIAnonymizerAndAlternativeRealizer.logLevel = 0;
		}

		boolean serialize = false;
		if (cmd.hasOption("serialize"))
		{
			serialize = true;
		}

		boolean deserialize = false;
		if (cmd.hasOption("deserialize"))
		{
			deserialize = true;
		}

		long time = System.currentTimeMillis();

		final GraphInterface graph = reader.readGraphFile();
		final int[] initSeq = DegreeSequence.calcDegreeSequence(graph);
		CompactDegreeSequence initialDegreeSequence = new CompactDegreeSequence(initSeq, true);

		long graphBuildTime;

		time = System.currentTimeMillis();

		Set<CompactDegreeSequence> solutions = null;

		if (!deserialize)
		{
			graphBuildTime = System.currentTimeMillis() - time;
			if (MainCLIAnonymizerAndAlternativeRealizer.logLevel > 0)
			{
				System.out.println("#Initial block degree sequence : " + initialDegreeSequence);
			}
			System.out.println("#Building graph and initial sequence done in : " + graphBuildTime);
			System.out.println("#Upperbound : "
					+ UpperBound.getFastUpperBoundForKAnon(initialDegreeSequence, MainCLIAnonymizerAndAlternativeRealizer.k).getS1());
			solutions = new DegreeAnonymizer(graph, initialDegreeSequence, MainCLIAnonymizerAndAlternativeRealizer.k, true, datareduction)
					.getAllkAnonRealSequences();
			final CompactDegreeSequence difference = solutions.iterator().next().difference(initialDegreeSequence);
			System.out.println("Anonymizing runtime " + ((System.currentTimeMillis() - time) / 1000d) + "sec. Anonymization cost is "
					+ difference.getTotalDegreeSum());
			MainCLIAnonymizerAndAlternativeRealizer.cost = difference.getTotalDegreeSum();
			System.out.println("cost: " + MainCLIAnonymizerAndAlternativeRealizer.cost);
			System.out.println("Amount of computed solutions : " + solutions.size());
		}

		final String outputFile = new File(cmd.getOptionValue("input-file")).getName() + "k" + MainCLIAnonymizerAndAlternativeRealizer.k
				+ (datareduction ? "ddr" : "noddr") + ".sequences";

		if (serialize)
		{

			ObjectOutputStream oos = null;
			ZipOutputStream zipout = null;
			try
			{
				final String outFilename = outputFile;

				zipout = new ZipOutputStream(new FileOutputStream(outFilename));
				zipout.putNextEntry(new ZipEntry("solutions"));
				oos = new ObjectOutputStream(zipout);
				oos.writeObject(solutions);
				zipout.putNextEntry(new ZipEntry("initial"));
				oos = new ObjectOutputStream(zipout);
				oos.writeObject(initialDegreeSequence);

			} catch (final IOException ex)
			{
				ex.printStackTrace();
			} finally
			{
				try
				{
					zipout.close();
					oos.close();
				} catch (final IOException ex)
				{
					ex.printStackTrace();
				}
			}
			System.out.println("Wrote solution sequences to disk.");
			System.exit(1);
		}

		else if (deserialize)
		{
			if (new File(outputFile).exists())
			{
				ZipInputStream zipin = null;
				try
				{
					zipin = new ZipInputStream(new FileInputStream(new File(outputFile)));
					ZipEntry zipentry = zipin.getNextEntry();
					while (zipentry != null)
					{
						if (zipentry.getName().compareTo("solutions") == 0)
						{
							final ObjectInputStream ois = new ObjectInputStream(zipin);
							solutions = (Set<CompactDegreeSequence>) ois.readObject();

						} else if (zipentry.getName().compareTo("initial") == 0)
						{
							final ObjectInputStream ois = new ObjectInputStream(zipin);
							initialDegreeSequence = (CompactDegreeSequence) ois.readObject();
						}

						zipentry = zipin.getNextEntry();
					}
					final CompactDegreeSequence difference = solutions.iterator().next().difference(initialDegreeSequence);
					MainCLIAnonymizerAndAlternativeRealizer.cost = difference.getTotalDegreeSum();
					System.out.println("Amount of computed solutions : " + solutions.size());
					System.out.println("cost: " + MainCLIAnonymizerAndAlternativeRealizer.cost);

				} catch (ClassNotFoundException | IOException ex)
				{
					System.out.println("Error while deserializing solution sequences file.");
					ex.printStackTrace();
				} finally
				{
					try
					{
						zipin.close();
					} catch (final IOException ex)
					{
						ex.printStackTrace();
					}
				}
			} else
			{
				System.out.println("Deserializing failed: No solution sequences file found");
				return null;
			}
		}

		final Set<Map<Integer, TIntSet>> edgeInsertionSets = new HashSet<>();
		Map<Integer, TIntSet> edgeInsertionSet = null;
		final Set<CompactDegreeSequence> solved = new HashSet<>();

		for (final Integer actualselectionStrategy : selectionStrategies)
		{
			for (final Integer actualProcessingStrategy : processingStrategies)
			{
				final long tempTime = System.currentTimeMillis();
				long solutionCounter = 0;
				for (final CompactDegreeSequence solution : solutions)
				{
					if ((MainCLIAnonymizerAndAlternativeRealizer.maxTimeInMin > 0)
							&& ((System.currentTimeMillis() - startintTime) > (60 * 1000 * MainCLIAnonymizerAndAlternativeRealizer.maxTimeInMin)))
					{ // Cancel after 30 Minutes
						System.out.println("Time exceeded.");
						final double constructionTime = (System.currentTimeMillis() - tempTime) / 1000.0;
						System.out.println("Selection strategy : "
								+ (actualselectionStrategy == GreedyGraphConstructor.SELECTION_STRATEGY_LEAST_COMMON_NEIGHBORS_REFINED ? "1 Enhanced"
										: actualselectionStrategy)
								+ " : Processing Strategy : "
								+ (actualProcessingStrategy == GreedyGraphConstructor.PROCESSING_STRATEGY_MAXDIFF_REFINED ? "1 Enhanced"
										: actualProcessingStrategy) + " : solved : " + solutionCounter + " of " + solutions.size() + " : time : "
								+ constructionTime);
						if (MainCLIAnonymizerAndAlternativeRealizer.out != null)
						{
							this.writeSolutionsToDisk(edgeInsertionSets);
						}
						return edgeInsertionSets;
					}
					edgeInsertionSet = new GreedyGraphConstructor(graph).constructGraph(initialDegreeSequence, solution,
							MainCLIAnonymizerAndAlternativeRealizer.simulate, actualProcessingStrategy, actualselectionStrategy);
					if (edgeInsertionSet != null)
					{
						if (solved.add(solution))
						{
							edgeInsertionSets.add(edgeInsertionSet);
						}
						solutionCounter++;
					}

				}
				final double constructionTime = (System.currentTimeMillis() - tempTime) / 1000.0;
				System.out.println("Selection strategy : "
						+ (actualselectionStrategy == GreedyGraphConstructor.SELECTION_STRATEGY_LEAST_COMMON_NEIGHBORS_REFINED ? "1 Refined\t"
								: actualselectionStrategy + "\t\t")
						+ " : Processing Strategy : "
						+ (actualProcessingStrategy == GreedyGraphConstructor.PROCESSING_STRATEGY_MAXDIFF_REFINED ? "1 Refined \t"
								: actualProcessingStrategy + "\t\t") + " : solved : " + solutionCounter + " of " + solutions.size() + " : time : "
						+ constructionTime);

			}

		}
		if (MainCLIAnonymizerAndAlternativeRealizer.out != null)
		{
			this.writeSolutionsToDisk(edgeInsertionSets);
		}
		return edgeInsertionSets;
	}

	public boolean writeSolutionsToDisk(final Set<Map<Integer, TIntSet>> edgeInsertionSets)
	{
		final boolean success = true;

		int counter = 1;
		final Iterator<Map<Integer, TIntSet>> it = edgeInsertionSets.iterator();
		while (it.hasNext())
		{

			final Map<Integer, TIntSet> set = it.next();
			final List<Edge> solutionSet = new LinkedList<>();
			final Set<Entry<Integer, TIntSet>> entry = set.entrySet();
			final Iterator<Entry<Integer, TIntSet>> entryIterator = entry.iterator();
			while (entryIterator.hasNext())
			{
				final Entry<Integer, TIntSet> edge = entryIterator.next();
				final int source = edge.getKey();
				final TIntSet targets = edge.getValue();
				final TIntIterator targetsIterator = targets.iterator();
				while (targetsIterator.hasNext())
				{
					final int target = targetsIterator.next();
					solutionSet.add(new Edge(source, target));
					solutionSet.add(new Edge(target, source));
				}
			}

			Collections.sort(solutionSet, new Comparator<Edge>()
			{

				@Override
				public int compare(final Edge o1, final Edge o2)
				{
					// TODO Auto-generated method stub
					return o1.getVertexA() - o2.getVertexB();
				}
			});

			BufferedWriter bw = null;

			try
			{
				bw = new BufferedWriter(new FileWriter(new File(MainCLIAnonymizerAndAlternativeRealizer.out + System.getProperty("file.separator")
						+ MainCLIAnonymizerAndAlternativeRealizer.instance + "_k" + MainCLIAnonymizerAndAlternativeRealizer.k + "_" + counter)));
				for (final Edge edge : solutionSet)
				{
					bw.write(edge.getVertexA() + " " + edge.getVertexB() + System.getProperty("line.separator"));
				}
			} catch (final IOException e)
			{

			} finally
			{
				try
				{
					bw.close();
				} catch (final IOException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

			counter++;

		}

		return success;
	}

}