package benchmarks;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;

import benchmarks.RunAllUpperBoundHeuristics.heuristic;
import reader.GraphReader;
import algorithm.DegreeAnonymizer;
import algorithm.dataStructures.GraphInterface;
import algorithm.graphRealizer.GraphRealizer;

public class PubliMain
{
	public static void main(final String[] args) throws FileNotFoundException, IOException
	{
		ClassLoader.getSystemClassLoader().setDefaultAssertionStatus(false);

		MainCLIAnonymizerAndAlternativeRealizer.logLevel = 0;
		DegreeAnonymizer.logDR = 0;
		GraphRealizer.log = 0;


		String[] paths = { "graphs/10thDimacs/", "graphs/webdata/" };

		int[] kValues = { 2, 3, 4, 5, 7, 9, 10, 15, 20, 30, 50, 100, 150, 200 };

		if (args.length < 1)
			System.out.println("Please specify the directory containing your graphs");
		else
			paths = new String[]{ args[0] };

		if (args.length >= 2)
		{
			kValues = new int[]{ Integer.parseInt(args[1]) };
		}
			

		
		{// write header
			LogStringBuilder.getLogger().printHeader(System.out);

			final FileOutputStream outStream = new FileOutputStream(new File("logFile.log"), true);
			LogStringBuilder.getLogger().printHeader(outStream);
			outStream.close();
		}

		for (final int k : kValues)
		{
			for (final String path : paths)
			{
				final File[] list = new File(path).listFiles();
				Arrays.sort(list);

				for (final File filePath : list)
				{
					if (filePath.isFile() && !filePath.isHidden())
					{
						MainCLIAnonymizerAndAlternativeRealizer.timeStamp = System.currentTimeMillis();

						LogStringBuilder.getLogger().cleanValues();
						LogStringBuilder.getLogger().startTime();

						final String fileName = filePath.getName();
						

						final GraphInterface graph = new GraphReader(path + fileName).readGraphFile();
						LogStringBuilder.getLogger().addTimeEntry("timeToLoadGraph");

						LogStringBuilder.getLogger().addEntry("file", fileName);
						LogStringBuilder.getLogger().addEntry("n", graph.getNumberOfVertices());
						LogStringBuilder.getLogger().addEntry("m", graph.getNumberOfEdges());
						LogStringBuilder.getLogger().addEntry("maxDeg", graph.getDelta());

						LogStringBuilder.getLogger().addEntry("k", k);

						for (heuristic heur : heuristic.values()) 
						{
							final RunOneHeuristic heuristicTester = new RunOneHeuristic(graph, k, heur);
							heuristicTester.start();
							try
							{
								heuristicTester.join(1000 * 60 * 5);
							} catch (final InterruptedException e1)
							{
							}
							if (heuristicTester.isAlive())
							{
								LogStringBuilder.getLogger().addEntry(RunAllUpperBoundHeuristics.getHeuristicLogField(heur), "aborted");
								heuristicTester.interrupt();
								heuristicTester.stop();
							}
							System.gc();
						}
						final TestAnonymizerCaller s = new TestAnonymizerCaller(graph, k, true);
						s.start();
						try
						{
							s.join(1000 * 60 * 5);

						} catch (final InterruptedException e)
						{

						}
						s.writeLogging();

						if (s.isAlive())
						{
							s.interrupt();
							s.stop();
						}

						{// printOutput
							LogStringBuilder.getLogger().printOutput(System.out);

							final FileOutputStream outStream = new FileOutputStream(new File("logFile.log"), true);
							LogStringBuilder.getLogger().printOutput(outStream);
							outStream.close();

							LogStringBuilder.getLogger().cleanValues();
						}
						System.gc();
					}
				}
			}
		}
	}
}
