package benchmarks;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;

import reader.GraphReader;
import algorithm.DegreeAnonymizer;
import algorithm.dataStructures.GraphInterface;
import algorithm.graphRealizer.GraphRealizer;

public class RunAllUpperBoundHeuristics 
{
	public enum heuristic 
	{ 
		CLUSTERING, FAST, LTBASED, SOPHISTICATED 
	}
	
	public static String getHeuristicLogField (heuristic heur)
	{
		if (heur == heuristic.CLUSTERING)
			return LogStringBuilder.LogKey_KOLJA_HEURISTIC;
		if (heur == heuristic.FAST)
			return LogStringBuilder.LogKey_FAST_UPPERBOUND_HEURISTIC;
		if (heur == heuristic.LTBASED)
			return LogStringBuilder.LogKey_SEMI_SOPHISTICATED_UPPERBOUND_HEURISTIC;
		if (heur == heuristic.SOPHISTICATED)
			return LogStringBuilder.LogKey_SOPHISTICATED_UPPERBOUND_HEURISTIC;
		return "";
	}
	
	public static void main(final String[] args) throws FileNotFoundException, IOException
	{
		ClassLoader.getSystemClassLoader().setDefaultAssertionStatus(false);

		MainCLIAnonymizerAndAlternativeRealizer.logLevel = 0;
		DegreeAnonymizer.logDR = 0;
		GraphRealizer.log = 0;

		String[] paths = { "graphs/10thDimacs/", "graphs/webdata/", "graphs/stanford/", "graphs/syntheticKAnon/" };

		int[] kValues = { 2, 3, 5, 10, 15, 20, 30, 50, 100, 150, 200, 4, 7, 9, 25, 40, 75 };

		{// write header
			LogStringBuilder.getLogger().printHeader(System.out);

			final FileOutputStream outStream = new FileOutputStream(new File("upperBound.log"), true);
			LogStringBuilder.getLogger().printHeader(outStream);
			outStream.close();
		}

		for (final String path : paths)
		{
			final File[] list = new File(path).listFiles();
			Arrays.sort(list);

			for (final File filePath : list)
			{
				if (filePath.isFile() && !filePath.isHidden())
				{
					LogStringBuilder.getLogger().startTime();
					final String fileName = filePath.getName();
					final GraphInterface graph = new GraphReader(path + fileName).readGraphFile();
					LogStringBuilder.getLogger().addTimeEntry("timeToLoadGraph");
					String timeToLoadGraph = LogStringBuilder.getLogger().getEntry("timeToLoadGraph");
							
					for (final int k : kValues)
					{
						MainCLIAnonymizerAndAlternativeRealizer.timeStamp = System.currentTimeMillis();

						LogStringBuilder.getLogger().cleanValues();
						LogStringBuilder.getLogger().startTime();

						LogStringBuilder.getLogger().addEntry("timeToLoadGraph", timeToLoadGraph);
						LogStringBuilder.getLogger().addEntry("file", fileName);
						LogStringBuilder.getLogger().addEntry("n", graph.getNumberOfVertices());
						LogStringBuilder.getLogger().addEntry("m", graph.getNumberOfEdges());
						LogStringBuilder.getLogger().addEntry("maxDeg", graph.getDelta());

						LogStringBuilder.getLogger().addEntry("k", k);

						for (heuristic heur : heuristic.values()) 
						{
							final RunOneHeuristic heuristicTester = new RunOneHeuristic(graph, k, heur);
							heuristicTester.start();
							try
							{
								heuristicTester.join(1000 * 60 * 5);
							} catch (final InterruptedException e1)
							{
							}
							if (heuristicTester.isAlive())
							{
								LogStringBuilder.getLogger().addEntry(getHeuristicLogField(heur), "aborted");
								heuristicTester.interrupt();
								heuristicTester.stop();
							}
							System.gc();
						}

						{// printOutput
							LogStringBuilder.getLogger().printOutput(System.out);

							final FileOutputStream outStream = new FileOutputStream(new File("upperBound.log"), true);
							LogStringBuilder.getLogger().printOutput(outStream);
							outStream.close();

							LogStringBuilder.getLogger().cleanValues();
						}
						System.gc();
					}
				}
			}
		}
	}

	
}
