package benchmarks;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;

import reader.GraphReader;
import algorithm.DegreeAnonymizer;
import algorithm.dataStructures.GraphInterface;
import algorithm.graphRealizer.GraphRealizer;

public class RunLowerBounds 
{
	public static void main(final String[] args) throws FileNotFoundException, IOException
	{
		ClassLoader.getSystemClassLoader().setDefaultAssertionStatus(false);

		MainCLIAnonymizerAndAlternativeRealizer.logLevel = 0;
		DegreeAnonymizer.logDR = 0;
		GraphRealizer.log = 0;

		int timelimit = 60; // timelimit for one instance in minutes
		String[] paths = { "graphs/10thDimacs/", "graphs/webdata/" };

		int[] kValues = { 2, 3, 4, 5, 7, 9, 10, 15, 20, 30, 50, 100, 200};

		if (args.length < 1 || args[0].equals("1"))
		{
			 paths = new String[]{ "graphs/10thDimacs/" };
		}
		else if (args[0].equals("2"))
		{
			 paths = new String[]{ "graphs/stanford/" };
		}
		else if (args[0].equals("3"))
		{
			 paths = new String[]{ "graphs/to_be_checked/" };
		}
		else if (args[0].equals("synthetic") && args.length >= 3)
		{
			 paths = new String[]{ "graphs/syntheticKAnon/" };
			 kValues = new int[]{ Integer.parseInt(args[1]) };
			 timelimit = Integer.parseInt(args[2]);
		}

		
		{// write header
			LogStringBuilder.getLogger().printHeader(System.out);

			final FileOutputStream outStream = new FileOutputStream(new File("lowerBound.log"), true);
			LogStringBuilder.getLogger().printHeader(outStream);
			outStream.close();
		}

		for (final String path : paths)
		{
			final File[] list = new File(path).listFiles();
			Arrays.sort(list);

			for (final File filePath : list)
			{
				if (filePath.isFile() && !filePath.isHidden())
				{
					LogStringBuilder.getLogger().startTime();
//					final String fileName = "coPapersCiteseer.graph";
					final String fileName = filePath.getName();
					final GraphInterface graph = new GraphReader(path + fileName).readGraphFile();
					LogStringBuilder.getLogger().addTimeEntry("timeToLoadGraph");
					String timeToLoadGraph = LogStringBuilder.getLogger().getEntry("timeToLoadGraph");
							
					for (final int k : kValues)
					{
						MainCLIAnonymizerAndAlternativeRealizer.timeStamp = System.currentTimeMillis();

						LogStringBuilder.getLogger().cleanValues();
						LogStringBuilder.getLogger().startTime();

						LogStringBuilder.getLogger().addEntry("timeToLoadGraph", timeToLoadGraph);

						LogStringBuilder.getLogger().addEntry("file", fileName);
						LogStringBuilder.getLogger().addEntry("n", graph.getNumberOfVertices());
						LogStringBuilder.getLogger().addEntry("m", graph.getNumberOfEdges());
						LogStringBuilder.getLogger().addEntry("maxDeg", graph.getDelta());

						LogStringBuilder.getLogger().addEntry("k", k);

						final TestAnonymizerCaller s = new TestAnonymizerCaller(graph, k, true);
						s.start();
						try
						{
							s.join(1000 * 60 * timelimit);

						} catch (final InterruptedException e)
						{

						}
						s.writeLogging();

						if (s.isAlive())
						{
							s.interrupt();
							s.stop();
						}

						{// printOutput
							LogStringBuilder.getLogger().printOutput(System.out);

							final FileOutputStream outStream = new FileOutputStream(new File("lowerBound.log"), true);
							LogStringBuilder.getLogger().printOutput(outStream);
							outStream.close();

							LogStringBuilder.getLogger().cleanValues();
						}
						System.gc();
					}
				}
			}
		}
	}

}
