package benchmarks;

import progOpt.MainFastUpperBoundHeuristic;
import progOpt.MainMoreSophisticatedUpperBoundHeuristic;
import algorithm.UpperBound;
import algorithm.dataStructures.GraphInterface;
import benchmarks.RunAllUpperBoundHeuristics.heuristic;

/**
 * Similar to {@code TestHeuristic} but runs only one selected heuristic 
 * @author nichterlein
 *
 */
public class RunOneHeuristic extends Thread
{	
	private final GraphInterface graph;

	private final int k;
	
	private final heuristic mode;

	public RunOneHeuristic(final GraphInterface graph, final int k, final heuristic mode)
	{
		this.graph = graph;
		this.k = k;
		this.mode = mode;
	}

	@Override
	public void run()
	{
		LogStringBuilder.getLogger().startTime();
		boolean verbose = false;
		final int maxTime = 60 * 5;

		if (this.mode == heuristic.CLUSTERING)
		{
			LogStringBuilder.getLogger().addEntry(LogStringBuilder.LogKey_KOLJA_HEURISTIC,
				UpperBound.getClusteringHeuristicUpperBound(this.k, this.graph), true);
		}
		else if (this.mode == heuristic.FAST)
		{
			MainFastUpperBoundHeuristic.setOptimizedParametersForFastUpperBound(maxTime, verbose);
			LogStringBuilder.getLogger().addEntry(LogStringBuilder.LogKey_FAST_UPPERBOUND_HEURISTIC,
					UpperBound.getFastUpperBoundForKAnonAndRealizable(this.graph, this.k, maxTime), true);
		}
		else if (this.mode == heuristic.LTBASED)
		{
			MainMoreSophisticatedUpperBoundHeuristic.setOptimizedParametersForMoreSophisticatedUpperBound(maxTime, verbose);
			LogStringBuilder.getLogger().addEntry(LogStringBuilder.LogKey_SEMI_SOPHISTICATED_UPPERBOUND_HEURISTIC,
					UpperBound.getSophisticatedUpperBoundForKAnonAndRealizable(this.graph, this.k, maxTime).getS1(), true);
		}
		else if (this.mode == heuristic.SOPHISTICATED)
		{
			MainMoreSophisticatedUpperBoundHeuristic.setOptimizedParametersForMoreSophisticatedUpperBound(maxTime, verbose);
			LogStringBuilder.getLogger().addEntry(LogStringBuilder.LogKey_SOPHISTICATED_UPPERBOUND_HEURISTIC,
					UpperBound.getMoreSophisticatedUpperBoundForKAnonAndRealizable(this.graph, this.k, maxTime).getS1(), true);
		}
	}
}
