package benchmarks;

import java.util.Set;

import dataStructures.IntTupel;
import dataStructures.Store;
import progOpt.MainMoreSophisticatedUpperBoundHeuristic;
import algorithm.DegreeAnonymizer;
import algorithm.dataStructures.CompactDegreeSequence;
import algorithm.dataStructures.DegreeSequence;
import algorithm.dataStructures.GraphInterface;
import algorithm.graphRealizer.CannotRealize;
import algorithm.graphRealizer.GraphRealizer;

public class TestAnonymizerCaller extends Thread
{
	public static final String LogKey_TEST_SOL_FOR_REALIZABLE = "#testSolForRealizable";

	public DegreeAnonymizer anonymizer;

	private final boolean applyDataReduction;

	private int foundRealizableSolution = -1;

	private final GraphInterface g;

	private final int k;

	public int testedToRealizeSolutions = 0;

	public TestAnonymizerCaller(final GraphInterface graph, final int k, final boolean applyDR)
	{
		this.g = graph;
		this.k = k;
		this.applyDataReduction = applyDR;

	}

	boolean applyTest(final boolean applyDataReduction, final GraphInterface graph, final int k)
	{
		boolean verbose = false;
		final int maxTime = 60;
		
		LogStringBuilder.getLogger().startTime();

		final CompactDegreeSequence initialDegreeSequence = new CompactDegreeSequence(DegreeSequence.calcDegreeSequence(graph), true);
		LogStringBuilder.getLogger().addTimeEntry("timeToComputeDegreeSequence");

		MainMoreSophisticatedUpperBoundHeuristic.setOptimizedParametersForMoreSophisticatedUpperBound(maxTime, verbose);

		
		this.anonymizer = new DegreeAnonymizer(graph, initialDegreeSequence, k, true, applyDataReduction);
		LogStringBuilder.getLogger().addTimeEntry("timeToIniDegreeAnonymizer");

		final Set<CompactDegreeSequence> solutions = this.anonymizer.getAllkAnonRealSequences();
		LogStringBuilder.getLogger().addTimeEntry(LogStringBuilder.LogKey_TIME_FOR_GET_ALL_K_ANON_REAL_SEQUENCES);

		final int numberOfSolutions = solutions.size();
		LogStringBuilder.getLogger().addEntry("numberOfSolutions", numberOfSolutions);

		final int degreeAnon2 = solutions.iterator().next().difference(initialDegreeSequence).getTotalDegreeSum();

		LogStringBuilder.getLogger().addEntry("solutionSize", degreeAnon2);

		LogStringBuilder.getLogger().addEntry("hopeforSuccussRate",this.anonymizer.getLastSolutionDegreeIterator().getHopeTestSuccessRate());
		LogStringBuilder.getLogger().addEntry("totalgetNoOfAEGHopeTests", this.anonymizer.getLastSolutionDegreeIterator().getNoOfAEGHopeTests());

		LogStringBuilder.getLogger().startTime();
		boolean foundRealizableSolution = false;
		for (final CompactDegreeSequence sol : solutions)
		{
			try
			{
				this.testedToRealizeSolutions++;
				Store<Integer, Iterable<IntTupel>> realization = GraphRealizer.isRealizable(graph, initialDegreeSequence, sol, k);
				if (realization.getS1() == degreeAnon2)
				{
					foundRealizableSolution = true;
					break;
				}
			} catch (final CannotRealize e)
			{
			}
		}
		LogStringBuilder.getLogger().addTimeEntry("timeForGraphRealizing");
		return foundRealizableSolution;
	}

	@Override
	public void run()
	{
		this.foundRealizableSolution = this.applyTest(this.applyDataReduction, this.g, this.k) ? 1 : 0;
	}

	public void writeLogging()
	{
		LogStringBuilder.getLogger().addEntry("realizable",
				this.foundRealizableSolution == 1 ? "true" : this.foundRealizableSolution == 0 ? "false" : "aborted");
		LogStringBuilder.getLogger().addEntry(TestAnonymizerCaller.LogKey_TEST_SOL_FOR_REALIZABLE, this.testedToRealizeSolutions);
		if (this.anonymizer != null)
		{
			LogStringBuilder.getLogger().addEntry("lastExaminedCost", this.anonymizer.lastExaminedCost + this.anonymizer.preprocessCost);
			LogStringBuilder.getLogger().addEntry("preprocessCost", this.anonymizer.preprocessCost);
			LogStringBuilder.getLogger().addEntry("lastUpperBoundInSolutionIterator", this.anonymizer.solutionDegreeIterator.getCurrentUpperbound());
		}
	}

}
