package progOpt;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.LineNumberReader;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;

import reader.GraphReader;
import algorithm.UpperBound;
import algorithm.dataStructures.CompactDegreeSequence;
import algorithm.dataStructures.DegreeSequence;
import algorithm.dataStructures.GraphInterface;
import algorithm.dataStructures.Timer;
import dataStructures.Store;

public class ComputeFeatureSetFile
{
	static DecimalFormat df2 = new DecimalFormat("#,###,###,##0.00###", new DecimalFormatSymbols(Locale.ENGLISH));

	public static String eraseKValueFromFileString(String filePath)
	{
		final int indexOf = filePath.indexOf("_k=");
		filePath = filePath.substring(0, indexOf) + filePath.substring(filePath.indexOf("_", indexOf + 2) + 1);
		return filePath;
	}

	public static void main(final String[] args) throws IOException
	{
		final LineNumberReader reader = new LineNumberReader(new FileReader(new File(args[0])));
		final FileWriter writer = new FileWriter(new File(args[1]));
		String s;
		writer.write("file\tedges\tnodes\tdensityPerc\tk2ClusteringHeuristic\tFastUpperBound\tFastUpperBoundTime"
				+ "\tSophisUpperBound\tSophisUpperBoundTime" + "\tdelta\tkanonUpper\tk\n");
		while ((s = reader.readLine()) != null)
		{
			String filePath = s;
			// String filePath =
			// s.replaceAll("/afs/akt.tu-berlin.de/user/sepphar/instancesForExperiments/",
			// "/home/sepphar/instancesForExperiments/");

			final int k = ComputeFeatureSetFile.parseKValueFromFileString(filePath);

			filePath = ComputeFeatureSetFile.eraseKValueFromFileString(filePath);

			System.out.print(s.replace("/afs/akt.tu-berlin.de/user/sepphar/instancesForExperiments/", ""));
			final GraphInterface graph = new GraphReader(MainFastUpperBoundHeuristic.replaceGraphFolderString(filePath)).readGraphFile();
			final long n = graph.getNumberOfVertices();
			final int m = graph.getNumberOfEdges();

			System.out.println(" read");
			writer.write(s);
			writer.write("\t" + m);
			writer.write("\t" + n);
			final long maxEdges = (n * (n - 1)) >> 1;
			writer.write("\t" + ComputeFeatureSetFile.df2.format((100d * m) / maxEdges));
			writer.write("\t" + UpperBound.getClusteringHeuristicUpperBound(k, graph));

			Timer timer = new Timer();
			writer.write("\t" + UpperBound.getFastUpperBoundForKAnonAndRealizable(graph, k, 60));
			writer.write("\t" + TimeLogger.timeFormatter.format(timer.elapsedTimeInSeconds()));

			timer = new Timer();
			writer.write("\t" + UpperBound.getSophisticatedUpperBoundForKAnonAndRealizable(graph, k, 60).getS1());
			writer.write("\t" + TimeLogger.timeFormatter.format(timer.elapsedTimeInSeconds()));

			writer.write("\t" + graph.getDelta());
			final Store<Integer, CompactDegreeSequence> upperBoundForKAnon = UpperBound.getFastUpperBoundForKAnon(new CompactDegreeSequence(
					DegreeSequence.calcDegreeSequence(graph), true), k);
			writer.write("\t" + upperBoundForKAnon.getS1());
			writer.write("\t" + k);

			writer.write("\n");
			writer.flush();
		}
		reader.close();
		writer.close();
	}

	public static int parseKValueFromFileString(final String filePath)
	{
		final int indexOf = filePath.indexOf("_k=");
		final int k = Integer.parseInt(filePath.substring(indexOf + 3, filePath.indexOf("_", indexOf + 2)));
		return k;
	}
}
