package progOpt;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.lang3.mutable.MutableDouble;

import reader.GraphReader;
import algorithm.SolutionDegreeIterator;
import algorithm.UpperBound;
import algorithm.dataStructures.GraphInterface;
import algorithm.dataStructures.Timer;
import algorithm.graphRealizer.GraphRealizer;
import algorithm.graphRealizer.GraphRealizerConfig;
import algorithm.graphRealizer.SophisticatedGraphRealizerConfigGenerator;

import commandLineOptionsHelpClasses.CLOption;
import commandLineOptionsHelpClasses.DoubleOpt;
import commandLineOptionsHelpClasses.IntOpt;
import commandLineOptionsHelpClasses.StringOpt;

public class MainFastUpperBoundHeuristic
{
	/**
	 * Max. number of running time seconds.
	 */
	public final static MutableDouble cutoff_time = new MutableDouble(300d);
	public static final MutableDouble Safety_Buffer_Fac_Max_Time_Realizer = new MutableDouble(0.9d);

	private static Collection<? extends CLOption> getAllFastkAnonHeuristicParams(final boolean verbose)
	{
		final ArrayList<CLOption> commandLineOptions = new ArrayList<CLOption>();
		commandLineOptions.add(new DoubleOpt(UpperBound.MAGIC_UPPERBOUND_HEURISTIC_CONSTANT, "MAGIC_UPPERBOUND_HEURISTIC_CONSTANT", verbose, false));

		commandLineOptions.add(new DoubleOpt(UpperBound.MAGIC_CONSTANT_FOR_COST_LMINUS1, "MAGIC_CONSTANT_FOR_COST_LMINUS1", verbose, false));
		commandLineOptions.add(new DoubleOpt(UpperBound.MAGIC_CONSTANT_FOR_COST_LMINUS1_EMPTY, "MAGIC_CONSTANT_FOR_COST_LMINUS1_EMPTY", verbose,
				false));
		commandLineOptions.add(new DoubleOpt(UpperBound.FastUpperBound_weightFrac1, "FastUpperBound_weightFrac1", verbose, false));
		return commandLineOptions;
	}

	public static Collection<CLOption> getAllGraphRealizerOptions(final boolean verbose)
	{
		final ArrayList<CLOption> commandLineOptions = new ArrayList<CLOption>();

		commandLineOptions.add(new IntOpt(GraphRealizerConfig.DIV_INNERITER_PARAM, "DIV_INNERITER_PARAM", verbose, false));
		commandLineOptions.add(new IntOpt(GraphRealizerConfig.DIV_MAXREORG_SIMPLE_PARAM_VALUE, "DIV_MAXREORG_SIMPLE_PARAM_VALUE", verbose, false));
		commandLineOptions.add(new IntOpt(GraphRealizerConfig.DIV_MAXREORG_ADV_PARAM_VALUE, "DIV_MAXREORG_ADV_PARAM_VALUE", verbose, false));
		commandLineOptions.add(new IntOpt(GraphRealizerConfig.DIV_OUTERITER_PARAM, "DIV_OUTERITER_PARAM", verbose, false));
		commandLineOptions.add(new IntOpt(GraphRealizerConfig.DIV_INNERLOOPMOSTPROMISING_PARAM, "DIV_INNERLOOPMOSTPROMISING_PARAM", verbose, false));

		commandLineOptions.add(new IntOpt(GraphRealizerConfig.MAX_INNERITER_PARAM, "MAX_INNERITER_PARAM", verbose, false));
		commandLineOptions.add(new IntOpt(GraphRealizerConfig.MAX_MAXREORG_PARAM_VALUE, "MAX_MAXREORG_PARAM_VALUE", verbose, false));
		commandLineOptions.add(new IntOpt(GraphRealizerConfig.MAX_OUTERITER_PARAM, "MAX_OUTERITER_PARAM", verbose, false));
		commandLineOptions.add(new IntOpt(GraphRealizer.MAX_INNERLOOP_ADD_MOST_PROMISING, "MAX_INNERLOOP_ADD_MOST_PROMISING", verbose, false));

		commandLineOptions.add(new DoubleOpt(GraphRealizer.PERCENTAGE_SHUFFLE_VERTEX_ORDER, "PERCENTAGE_SHUFFLE_VERTEX_ORDER", verbose, false));
		commandLineOptions.add(new IntOpt(GraphRealizer.PERCENTAGE_DISTURB_VERTEX_DEGREE_ORDER, "PERCENTAGE_DISTURB_VERTEX_DEGREE_ORDER", verbose,
				false));

		commandLineOptions.add(new IntOpt(GraphRealizer.SHUFFLE_DEGREE_VERTEX_MAPPING, "SHUFFLE_DEGREE_VERTEX_MAPPING", verbose, false));

		commandLineOptions.add(new DoubleOpt(SophisticatedGraphRealizerConfigGenerator.InnerIterTOOuterIter_Frac, "InnerIterTOOuterIter_Frac",
				verbose, false));
		// "InnerIterTOOuterIter_Frac"
		commandLineOptions.add(new DoubleOpt(SophisticatedGraphRealizerConfigGenerator.maxReorgTOInnerIter_Frac, "maxReorgTOInnerIter_Frac", verbose,
				false));
		commandLineOptions.add(new DoubleOpt(SophisticatedGraphRealizerConfigGenerator.maxSimpleReorgTOmaxAdvReorg_Frac,
				"maxSimpleReorgTOmaxAdvReorg_Frac", verbose, false));

		commandLineOptions.add(new DoubleOpt(SophisticatedGraphRealizerConfigGenerator.timeThresholdForIncrease, "timeThresholdForIncrease", verbose,
				false));
		commandLineOptions.add(new DoubleOpt(SophisticatedGraphRealizerConfigGenerator.timeThresholdForIncreaseRandom,
				"timeThresholdForIncreaseRandom", verbose, false));

		commandLineOptions.add(new DoubleOpt(SophisticatedGraphRealizerConfigGenerator.Config_Generator_HighQuality_Improve_Fac,
				"Config_Generator_HighQuality_Improve_Fac", verbose, false));

		commandLineOptions.add(new IntOpt(UpperBound.StepIncrease_Magic_Const, "StepIncrease_Magic_Const", verbose, false));
		commandLineOptions.add(new DoubleOpt(UpperBound.StepIncrease_Magic_Const2, "StepIncrease_Magic_Const2", verbose, false));
		commandLineOptions.add(new IntOpt(UpperBound.StepIncrease_Magic_Const3, "StepIncrease_Magic_Const3", verbose, false));
		commandLineOptions.add(new IntOpt(UpperBound.HIGH_QUALITY_IN_EACH_STEP, "HIGH_QUALITY_IN_EACH_STEP", verbose, false));

		commandLineOptions.add(new DoubleOpt(SophisticatedGraphRealizerConfigGenerator.SOPHISTICATED_CONFIG_GENERATOR_UPDATE_NONE_VALUABLE_STATISTIC,
				"SOPHISTICATED_CONFIG_GENERATOR_UPDATE_NONE_VALUABLE_STATISTIC", verbose, false));

		commandLineOptions.add(new IntOpt(UpperBound.REALIZE_SOLUTIONS_WEIGHT_OF_FIRST, "REALIZE_SOLUTIONS_WEIGHT_OF_FIRST", verbose, false));
		commandLineOptions.add(new DoubleOpt(MainFastUpperBoundHeuristic.Safety_Buffer_Fac_Max_Time_Realizer, "Safety_Buffer_Fac_Max_Time_Realizer",
				verbose, false));
		commandLineOptions
				.add(new DoubleOpt(GraphRealizer.Safety_Buffer_Fac_MAX_Time_Configer, "Safety_Buffer_Fac_MAX_Time_Configer", verbose, false));
		commandLineOptions.add(new IntOpt(GraphRealizer.TIME_FACTOR_AS_LONG_AS_NO_SOLUTION_FOUND, "TIME_FACTOR_AS_LONG_AS_NO_SOLUTION_FOUND",
				verbose, false));
		commandLineOptions.add(new DoubleOpt(SophisticatedGraphRealizerConfigGenerator.maxBound_multFactor, "maxBound_multFactor", verbose, false));
		commandLineOptions.add(new DoubleOpt(SophisticatedGraphRealizerConfigGenerator.MULTFACTOR_INCREASE_COST, "MULTFACTOR_INCREASE_COST", verbose,
				false));
		commandLineOptions.add(new DoubleOpt(SophisticatedGraphRealizerConfigGenerator.MULTFACTOR_DECREASE_COST, "MULTFACTOR_DECREASE_COST", verbose,
				false));

		commandLineOptions.add(new IntOpt(algorithm.CostWaster.COSTWASTER_UPPERBOUND_NO_SOLUTIONS, "COSTWASTER_UPPERBOUND_NO_SOLUTIONS", verbose,
				false));

		return commandLineOptions;
	}

	private static Collection<? extends CLOption> getAllMoreSophisticatedkAnonHeuristicParams(final boolean verbose)
	{
		// TODO
		final ArrayList<CLOption> commandLineOptions = new ArrayList<CLOption>();

		commandLineOptions.add(new IntOpt(SolutionDegreeIterator.noOfStoredSolutions, "SOLUTION_ITERATOR_noOfStoredSolutions", verbose, false));
		commandLineOptions.add(new IntOpt(SolutionDegreeIterator.constantHowOftenHopeForIsApplied,
				"SOLUTION_ITERATOR_constantHowOftenHopeForIsApplied", verbose, false));
		commandLineOptions.add(new IntOpt(SolutionDegreeIterator.constantAdvancedInHope, "SOLUTION_ITERATOR_constantAdvancedInHope", verbose, false));

		commandLineOptions.add(new IntOpt(SolutionDegreeIterator.applyHopeTestAbsoluteLowerBound,
				"SOLUTION_ITERATOR_applyHopeTestAbsoluteLowerBound", verbose, false));

		commandLineOptions
				.add(new IntOpt(SolutionDegreeIterator.applyHopeTestLowerBound, "SOLUTION_ITERATOR_applyHopeTestLowerBound", verbose, false));
		commandLineOptions.add(new IntOpt(SolutionDegreeIterator.hopeTestApplyMode, "SOLUTION_ITERATOR_hopeTestApplyMode", verbose, false));
		commandLineOptions
				.add(new IntOpt(SolutionDegreeIterator.maxHopeTestsInOneBranch, "SOLUTION_ITERATOR_maxHopeTestsInOneBranch", verbose, false));

		return commandLineOptions;
	}

	private static Collection<? extends CLOption> getAllSophisticatedkAnonHeuristicParams(final boolean verbose)
	{
		// TODO
		final ArrayList<CLOption> commandLineOptions = new ArrayList<CLOption>();
		return commandLineOptions;
	}

	private static Collection<? extends CLOption> getAllStandardSMACOptions(final StringBuilder fileNameB, final boolean verbose)
	{
		final ArrayList<CLOption> commandLineOptions = new ArrayList<CLOption>();
		if (fileNameB != null)
		{
			commandLineOptions.add(new StringOpt(fileNameB, "instance_name", 0, verbose, true));
		}
		// commandLineOptions.add(new StringOpt(new StringBuilder(), "instance_specific infor", 1, verbose, true));
		commandLineOptions.add(new DoubleOpt(MainFastUpperBoundHeuristic.cutoff_time, "cutoff_time", 2, verbose, true));
		// commandLineOptions.add(new DoubleOpt(new MutableDouble(), "cutoff_length", 3, verbose, true));
		return commandLineOptions;
	}

	public static void main(final String[] args)
	{
		final Timer timer = new Timer();
		final boolean verbose = false;
		final StringBuilder fileNameB = new StringBuilder();

		// reads parameters from command line
		MainFastUpperBoundHeuristic.setAndReadCommandLineParameters(args, verbose, fileNameB, true, true, false, false);

		 // sets optimized values, completly ignores command line!!!
//		 MainFastUpperBoundHeuristic.setOptimizedParametersForFastUpperBound(60, verbose);

		String filePath = fileNameB.toString();
		final int k = ComputeFeatureSetFile.parseKValueFromFileString(filePath);
		filePath = ComputeFeatureSetFile.eraseKValueFromFileString(filePath);
		System.out.println(filePath);
		System.out.println("k=" + k);
		filePath = MainFastUpperBoundHeuristic.replaceGraphFolderString(filePath);

		final GraphInterface graph = new GraphReader(filePath).readGraphFile();

		final long startTimeStamp = System.currentTimeMillis();

		final int maxTime = (int) Math.floor(MainFastUpperBoundHeuristic.Safety_Buffer_Fac_Max_Time_Realizer.doubleValue()
				* (MainFastUpperBoundHeuristic.cutoff_time.doubleValue() - timer.elapsedTimeInSeconds()));
		final int upperBound = UpperBound.getFastUpperBoundForKAnonAndRealizable(graph, k, maxTime);
		System.out.println("upperbound:" + upperBound);
		System.out.flush();

		final double time = (System.currentTimeMillis() - startTimeStamp) / 1000d;
		System.out.println("heuristic time:" + time);
		System.out.println("total time:" + timer.elapsedTimeInSeconds());
		System.out.flush();
	}

	public static String replaceGraphFolderString(final String filePath)
	{
		// return filePath.replaceAll("/afs/akt.tu-berlin.de/user/sepphar/instancesForExperiments/", "/home/sepphar/instancesForExperiments/");
		return filePath;
	}

	/**
	 * Reads the parameters from {@code args} and sets them.
	 * 
	 * @param args
	 * @param verbose
	 * @param fileNameB
	 * @param addGraphRealizerParams
	 * @param addFastKAnonParams
	 * @param addSophisticatedkAnonParams
	 * @param addMoreSophisticatedkAnonParams
	 */
	public static void setAndReadCommandLineParameters(final String[] args, final boolean verbose, final StringBuilder fileNameB,
			final boolean addGraphRealizerParams, final boolean addFastKAnonParams, final boolean addSophisticatedkAnonParams,
			final boolean addMoreSophisticatedkAnonParams)
	{
		final ArrayList<CLOption> commandLineOptions = new ArrayList<CLOption>();

		// add all default options
		commandLineOptions.addAll(MainFastUpperBoundHeuristic.getAllStandardSMACOptions(fileNameB, verbose));

		if (addGraphRealizerParams)
		{// add all options for graph realizer
			commandLineOptions.addAll(MainFastUpperBoundHeuristic.getAllGraphRealizerOptions(verbose));
		}
		if (addFastKAnonParams)
		{// add all fast kAnonHeuristicParams
			commandLineOptions.addAll(MainFastUpperBoundHeuristic.getAllFastkAnonHeuristicParams(verbose));
		}

		if (addSophisticatedkAnonParams)
		{// add all sophisticated kAnonHeuristicParams
			commandLineOptions.addAll(MainFastUpperBoundHeuristic.getAllSophisticatedkAnonHeuristicParams(verbose));
		}

		if (addMoreSophisticatedkAnonParams)
		{// add all sophisticated kAnonHeuristicParams
			commandLineOptions.addAll(MainFastUpperBoundHeuristic.getAllMoreSophisticatedkAnonHeuristicParams(verbose));
		}
		for (final CLOption k : commandLineOptions)
		{
			k.setValue(args);
		}
	}

	/**
	 * Sets the parameters optimized by SMAC.
	 * 
	 * @param verbose
	 * @param fileNameB
	 * @param addGraphRealizerParams
	 * @param addFastKAnonParams
	 * @param addSophisticatedkAnonParams
	 * @param addMoreSophisticatedkAnonParams
	 */
	public static void setOptimizedParametersForFastUpperBound(final int maxRunTime, final boolean verbose)
	{

		final boolean addGraphRealizerParams = true;
		final boolean addFastKAnonParams = true;
		final boolean addSophisticatedkAnonParams = false;
		final boolean addMoreSophisticatedkAnonParams = false;

		final String args = maxRunTime
				+ " "
				+ maxRunTime
				+ " "
				+ maxRunTime
				+ " "
				+ maxRunTime
				+ " "
				+ "-COSTWASTER_UPPERBOUND_NO_SOLUTIONS 2 -Config_Generator_HighQuality_Improve_Fac 2.0473333889907877 -DIV_INNERITER_PARAM 33607 -DIV_INNERLOOPMOSTPROMISING_PARAM 21076 -DIV_MAXREORG_ADV_PARAM_VALUE 918987 -DIV_MAXREORG_SIMPLE_PARAM_VALUE 539549 -DIV_OUTERITER_PARAM 158447 -FastUpperBound_weightFrac1 0.8636189914820949 -HIGH_QUALITY_IN_EACH_STEP 4810 -InnerIterTOOuterIter_Frac 6.336195788622101 -MAGIC_CONSTANT_FOR_COST_LMINUS1 1.3607502667606555 -MAGIC_CONSTANT_FOR_COST_LMINUS1_EMPTY 0.9004955079826461 -MAGIC_UPPERBOUND_HEURISTIC_CONSTANT 2.6310269985765284 -MAX_INNERITER_PARAM 805 -MAX_INNERLOOP_ADD_MOST_PROMISING 58 -MAX_MAXREORG_PARAM_VALUE 1270 -MAX_OUTERITER_PARAM 4481 -MULTFACTOR_DECREASE_COST 0.5245428912252826 -MULTFACTOR_INCREASE_COST 0.41040113178338966 -PERCENTAGE_DISTURB_VERTEX_DEGREE_ORDER 291 -PERCENTAGE_SHUFFLE_VERTEX_ORDER 0.010275499150166123 -REALIZE_SOLUTIONS_WEIGHT_OF_FIRST 1 -SHUFFLE_DEGREE_VERTEX_MAPPING 2 -SOPHISTICATED_CONFIG_GENERATOR_UPDATE_NONE_VALUABLE_STATISTIC 0.632106096179963 -SOPHISTICATED_CONFIG_GENERATOR_UPDATE_VALUABLE_STATISTIC 0.832036445334212 -Safety_Buffer_Fac_MAX_Time_Configer 0.8668877904566474 -Safety_Buffer_Fac_Max_Time_Realizer 0.978010794043207 -StepIncrease_Magic_Const 13 -StepIncrease_Magic_Const2 2.6779902388821912 -StepIncrease_Magic_Const3 13 -TIME_FACTOR_AS_LONG_AS_NO_SOLUTION_FOUND 4 -maxBound_multFactor 1.0600730960610822 -maxReorgTOInnerIter_Frac 1.7999999999999998 -maxSimpleReorgTOmaxAdvReorg_Frac 5.4653598733058795 -timeThresholdForIncrease 1.2373870785255843 -timeThresholdForIncreaseRandom 1.1683180129575088";
		MainFastUpperBoundHeuristic.setAndReadCommandLineParameters(args.split("\\s"), verbose, null, addGraphRealizerParams, addFastKAnonParams,
				addSophisticatedkAnonParams, addMoreSophisticatedkAnonParams);
	}
}
