package progOpt;

import reader.GraphReader;
import algorithm.UpperBound;
import algorithm.dataStructures.GraphInterface;
import algorithm.dataStructures.Timer;

public class MainMoreSophisticatedUpperBoundHeuristic
{
	public static void main(final String[] args)
	{
		final Timer timer = new Timer();
		final boolean verbose = false;
		final StringBuilder fileNameB = new StringBuilder();

		// reads parameters from command line
		MainFastUpperBoundHeuristic.setAndReadCommandLineParameters(args, verbose, fileNameB, true, false, false, true);

		// sets optimized values, completly ignores command line!!!
		// MainMoreSophisticatedUpperBoundHeuristic.setOptimizedParametersForMoreSophisticatedUpperBound(60, verbose);

		String filePath = fileNameB.toString();
		final int k = ComputeFeatureSetFile.parseKValueFromFileString(filePath);
		filePath = ComputeFeatureSetFile.eraseKValueFromFileString(filePath);
		System.out.println(filePath);
		System.out.println("k=" + k);
		filePath = MainFastUpperBoundHeuristic.replaceGraphFolderString(filePath);

		final GraphInterface graph = new GraphReader(filePath).readGraphFile();

		final long startTimeStamp = System.currentTimeMillis();

		// System.out.println("clustering heuristik:" +
		// UpperBound.getClusteringHeuristicUpperBound(k, graph) + "("
		// + (System.currentTimeMillis() - startTimeStamp) / 1000d + ")");
		// startTimeStamp = System.currentTimeMillis();

		// DegreeAnonymizer2.logDR = 1;
		// UpperBound.printRealizerParams = true;
		// // GraphRealizer.log = 1;
		final int maxTime = (int) Math.floor(MainFastUpperBoundHeuristic.Safety_Buffer_Fac_Max_Time_Realizer.doubleValue()
				* (MainFastUpperBoundHeuristic.cutoff_time.doubleValue() - timer.elapsedTimeInSeconds()));
		final int upperBound = UpperBound.getMoreSophisticatedUpperBoundForKAnonAndRealizable(graph, k, maxTime).getS1();
		System.out.println("upperbound:" + upperBound);
		System.out.flush();

		final double time = (System.currentTimeMillis() - startTimeStamp) / 1000d;
		System.out.println("heuristic time:" + time);
		System.out.println("total time:" + timer.elapsedTimeInSeconds());
		System.out.flush();
	}

	/**
	 * Sets the parameters optimized by SMAC.
	 * 
	 * @param verbose
	 * @param fileNameB
	 * @param addGraphRealizerParams
	 * @param addFastKAnonParams
	 * @param addSophisticatedkAnonParams
	 * @param addMoreSophisticatedkAnonParams
	 */
	public static void setOptimizedParametersForMoreSophisticatedUpperBound(final int maxRunTime, final boolean verbose)
	{

		final boolean addGraphRealizerParams = true;
		final boolean addFastKAnonParams = false;
		final boolean addSophisticatedkAnonParams = false;
		final boolean addMoreSophisticatedkAnonParams = true;

		final String args = maxRunTime
				+ " "
				+ maxRunTime
				+ " "
				+ maxRunTime
				+ " "
				+ maxRunTime
				+ " "
				+ "-COSTWASTER_UPPERBOUND_NO_SOLUTIONS 4 -Config_Generator_HighQuality_Improve_Fac 7.0 -DIV_INNERITER_PARAM 200000 "
				+ "-DIV_INNERLOOPMOSTPROMISING_PARAM 7284 -DIV_MAXREORG_ADV_PARAM_VALUE 113400 -DIV_MAXREORG_SIMPLE_PARAM_VALUE 1004498 "
				+ "-DIV_OUTERITER_PARAM 12716 -HIGH_QUALITY_IN_EACH_STEP 20 -InnerIterTOOuterIter_Frac 1.1 -MAX_INNERITER_PARAM 9 "
				+ "-MAX_INNERLOOP_ADD_MOST_PROMISING 9 -MAX_MAXREORG_PARAM_VALUE 100 -MAX_OUTERITER_PARAM 8 -MULTFACTOR_DECREASE_COST 0.5 "
				+ "-MULTFACTOR_INCREASE_COST 0.8 -PERCENTAGE_DISTURB_VERTEX_DEGREE_ORDER 3 -PERCENTAGE_SHUFFLE_VERTEX_ORDER 0.3 -REALIZE_SOLUTIONS_WEIGHT_OF_FIRST 2 "
				+ "-SHUFFLE_DEGREE_VERTEX_MAPPING 2 "
//				+ "-SOLUTION_ITERATOR_applyHopeTestAbsoluteLowerBound 78 -SOLUTION_ITERATOR_applyHopeTestLowerBound 48 "
//				+ "-SOLUTION_ITERATOR_constantAdvancedInHope 55 -SOLUTION_ITERATOR_constantHowOftenHopeForIsApplied 5 -SOLUTION_ITERATOR_hopeTestApplyMode 3 "
//				+ "-SOLUTION_ITERATOR_noOfStoredSolutions 2000 "
				+ "-SOPHISTICATED_CONFIG_GENERATOR_UPDATE_NONE_VALUABLE_STATISTIC 0.38 "
				+ "-SOPHISTICATED_CONFIG_GENERATOR_UPDATE_VALUABLE_STATISTIC 0.75 -Safety_Buffer_Fac_MAX_Time_Configer 0.89 "
				+ "-Safety_Buffer_Fac_Max_Time_Realizer 0.97 -StepIncrease_Magic_Const 2 -StepIncrease_Magic_Const2 0.8999999999999999 "
				+ "-StepIncrease_Magic_Const3 10 -TIME_FACTOR_AS_LONG_AS_NO_SOLUTION_FOUND 4 -maxBound_multFactor 1.28 -maxReorgTOInnerIter_Frac 1.1 "
				+ "-maxSimpleReorgTOmaxAdvReorg_Frac 2.2 -timeThresholdForIncrease 1.1 -timeThresholdForIncreaseRandom 1.1";
		MainFastUpperBoundHeuristic.setAndReadCommandLineParameters(args.split("\\s"), verbose, null, addGraphRealizerParams, addFastKAnonParams,
				addSophisticatedkAnonParams, addMoreSophisticatedkAnonParams);
	}

}
